<?php
namespace Teknisa\OPL\Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DTO\Request\Row;
use \Zeedhi\Framework\DTO\Response\Message;
use \Zeedhi\Framework\DTO\Response;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource;
use Exception;
use \Teknisa\OPL\Services\Dfs48000SaidaDoOperadorLogisticoService;
    
class Dfs48000SaidaDoOperadorLogisticoController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    
    public function __construct($crudService, $parameterBag, Dfs48000SaidaDoOperadorLogisticoService $service) {
        $this->crudService  = $crudService;
        $this->service      = $service;
        $this->parameterBag = $parameterBag;
    }
    
    public function getDtProcessaOpl(Row $request, Response $response) {
        try {
            $params = $request->getRow();
      
            $result = $this->service->getDtProcessaOpl($params);
            $dtprocessa = $this->service->getDtProcessaAtual($params);
            
            $result[] = isset($dtprocessa) ? $dtprocessa[0] : null;
            
            $response->addDataSet(new DataSet('res', $result));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function validaItensPedido(Row $request, Response $response) {
        try {
            $rows = $request->getRow();
            $dados = $rows['DADOSGERACAO'];
            $pedidos = $rows['NFS'];
            $retornos = array();
            $pedComErro = array();
            $pedSemErro = array();
            $pedidosParaSaida = array();
            $retornoSaida;
            
            $dtprocessa = $this->service->getDtProcessaAtual($dados);
            $stwebservice = $this->service->getStWebService($dados);
            
            if(!empty($stwebservice)) {
                $dados['WEBSERVICE'] = 'S';
            } else {
                $dados['WEBSERVICE'] = 'N';
            }
            
            $dados['DTPROC'] = $dtprocessa[0]['DTPROCESSA'];
            
            foreach($pedidos as $ped) {
                $result = $this->service->validaItensPedido($ped, $dados);
                
                if(!empty($result)) {
                    foreach($result as $res) {
                        if($res['LISTAERRO'] != NULL) {
                            $strErro = '';
                            $codErros = explode('_', $res['LISTAERRO']);
                            $codErros = array_filter($codErros, static function($e) {
                                return $e != "";
                            });
                            
                            foreach($codErros as $err) {
                                if($strErro != '')
                                    $strErro .= "\n";
                                    
                                switch($err) {
                                    case 'PRODINATIVO':
                                        $strErro .= "Produto Inativo para o OL.";
                                        break;
                                    case 'ERROVLR':
                                        $strErro .= "Produto com Custo menor ou igual a zero.";
                                        break;
                                    case 'ERRORES':
                                        $strErro .= "Produto sem Reserva.";
                                        break;
                                    case 'ERROQTD':
                                        $strErro .= "Quantidade atendida pelo OL deve ser informada.";
                                        break;
                                    case 'ERRORELAC':
                                        $strErro .= "Produto sem relacionamento com produto do OL.";
                                        break;
                                    case 'ERROEST':
                                        $strErro .= "Sem quantidade disponível no estoque do OL.";
                                        break;
                                    case 'ERROCX':
                                        $strErro .= "Produto caixa inativo.";
                                        break;
                                }
                            }
                            
                            $res['NMERRO'] = $strErro;
                            
                            $retornos[] = $res;
                            
                            if(!in_array($res['NRAUTOFORN'], $pedComErro)) {
                                $pedComErro[] = $res['NRAUTOFORN'];
                            }
                        }
                    }
                    
                    if(count($pedComErro) > 0) {
                        foreach($pedidos as $ped) {
                            if(!in_array($ped['NRAUTOFORN'], $pedComErro)) {
                                if(!in_array($ped['NRAUTOFORN'], $pedSemErro)) {
                                    $pedSemErro[] = $ped['NRAUTOFORN'];
                                }
                            }
                        }
                    }
                }
            }
            
            if(count($pedSemErro)) {
                foreach($pedSemErro as $pse) {
                    foreach($pedidos as $ped) {
                        if($ped['NRAUTOFORN'] == $pse) {
                            $pedidosParaSaida[] = $ped;
                        }
                    }
                }
            }
            
            $teste = $this->service->gerarSaida($pedidos, $dados);
            
            var_dump($teste);
            
            if(count($pedidosParaSaida) > 0) {
                // chama toda parte de geracao de notas para os pedidos sem erros
                $retornoSaida = $this->gerarSaidaPedidos($pedidosParaSaida, $dados);
            } else {
                $retornoSaida = array(
                    'ERROS'         => array(),
                    'NOTAS_GERADAS' => array()
                );
            }
            
            $response->addDataSet(new DataSet('saida', $retornoSaida));
            $response->addDataSet(new DataSet('pedErros', $retornos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function gerarSaidaPedidos($pedidos, $dados) {
        $retorno = array(
            'ERROS'         => array(),
            'NOTAS_GERADAS' => array()
        );
        
        return $retorno;
    }
}
