<?php

namespace Teknisa\OPL\Factories;

/* Bibliotecas */
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Model\Fiscal\Entities\Arqxmlnfe;
use Teknisa\Model\Fiscal\Entities\Notadestinada;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Remote\FileUploader\Server as fileUploader;

class ManifestacaoDestinatarioFactory {
    
    const CONTADOR_NOVOCODIGO_LOTEMANIFESTACAO = 'LOTEMANIFESTACAO';
    const CONTADOR_NOVOCODIGO_NOTAMANIFESTACAO = 'NOTAMANIFESTACAO';
    const TAMANHO_LOTENFE = 15;
    
    protected $entityManager;
    protected $fileUploader;
    protected $environment;

    public function __construct(EntityManager $entityManager, $environment, $novoCodigo) {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->novoCodigo    = $novoCodigo;
        // $this->fileUploader  = $fileUploader;
        set_time_limit(0);
    }
    
    public function getDataSincronizacao($cnpj) {
        $sql = "SELECT (TO_CHAR(MAX(DTSINCRONIZACAO), 'dd/mm/yyyy hh24:mi:ss')) 
                    DTSINCRONIZACAO
                    FROM SINCRONIZACAONFE
                   WHERE NRINSJURIEST = :P_NRINSJURIEST";
    
        $params = array(
            "P_NRINSJURIEST"          => $cnpj
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params)[0];
    }
    
    public function calculaHora($data) {
        $sql = "SELECT 45                                   - TO_NUMBER(TRUNC(MOD(T1.DIFERENCA*24*60, 60))) MINUTOS
                FROM
                  (SELECT TO_DATE(T0.DATA_FINAL, 'dd/mm/yyyy hh24:mi:ss') - TO_DATE(T0.DATA_INICIAL, 'dd/mm/yyyy hh24:mi:ss') DIFERENCA
                  FROM
                    (SELECT MAX(TO_CHAR(TO_DATE(:P_DATA, 'dd/mm/yyyy         hh24:mi:ss'), 'dd/mm/yyyy hh24:mi:ss')) DATA_INICIAL,
                      TO_CHAR(SYSDATE, 'dd/mm/yyyy hh24:mi:ss') DATA_FINAL
                    FROM SINCRONIZACAONFE
                    ) T0
                  ) T1";
                  
        $params = array(
            "P_DATA"   => $data
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params)[0];
    }
    
    public function getDadosWebService($tpAmb) {
        $sql = "SELECT WSRECEPCAO,
                 WSRETRECEPCAO,
                 WSEVENTO,
                 WSCONSULTACAD,
                 WSSTATUSSERVICO,
                 WSINUTILIZACAO,
                 WSCONSULTAPRO,
                 WSAUTORIZACAO,
                 WSRETAUTORIZACAO,
                 WSDOWNLOADNF,
                 WSCONSULTADEST,
                 CDSVC,
                 WSDISTRIBUICAODFE
            FROM WEBSERVICENA
           WHERE CDPAIS     = :P_CDPAIS
             AND IDSERVICO  = :P_IDSERVICO
             AND IDAMBIENTE = :P_IDAMBIENTE";
         
        $params = array(
            "P_CDPAIS"    => '0055',
            "P_IDSERVICO" => 'NF',
            "P_IDAMBIENTE" => $tpAmb
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params)[0];
    }
    
    public function insereHoraSincronizacao($nrInjusriest, $nrSincronizacaoNfe, $cdOperador, $nrUltimoNsu, $nrOrg, $tpAmb) {
        $sql = "INSERT INTO SINCRONIZACAONFE
                     (NRINSJURIEST,
                      NRSINCRONIZANFE,
                      DTSINCRONIZACAO,
                      CDOPERADOR,
                      NRULTIMONSU,
                      NRORG,
                      IDAMBIENTENFE)
                  VALUES
                     (:P_NRINSJURIEST,
                      :P_NRSINCRONIZANFE,
                      SYSDATE,
                      :P_CDOPERADOR,
                      :P_NRULTIMONSU,
                      :P_NRORG,
                      :P_IDAMBIENTENFE)";
        
        $params = ["P_NRINSJURIEST"    => $nrInjusriest,
                   "P_NRSINCRONIZANFE" => $nrSincronizacaoNfe,
                   "P_CDOPERADOR"      => $cdOperador,
                   "P_NRULTIMONSU"     => $nrUltimoNsu,
                   "P_NRORG"           => $nrOrg,
                   "P_IDAMBIENTENFE"   => $tpAmb];

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function existeNotaDestinada($nrAcessoNfe) {
        $sql = "SELECT IDSTATUSMANIF
                FROM NOTADESTINADA
                WHERE NRACESSONFE = :P_NRACESSONFE";
        
        $params = array(
            "P_NRACESSONFE"    => $nrAcessoNfe
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    /*public function atualizaNotaDestinada($idStatusNfe, $idStatusManif, $dtRecbto, $nrAcessoNfe) {
        $sql = "UPDATE NOTADESTINADA
            SET IDSTATUSNFE   = :P_IDSTATUSNFE,
              IDSTATUSMANIF   = NVL(:P_IDSTATUSMANIF, IDSTATUSMANIF),
              DTRECBTO        = :P_DTRECBTO
            WHERE NRACESSONFE = :P_NRACESSONFE";
            
        $params = array(
            "P_IDSTATUSNFE"     => $idStatusNfe,
            "P_IDSTATUSMANIF"   => $idStatusManif,
            "P_DTRECBTO"        => $dtRecbto,
            "P_NRACESSONFE"     => $nrAcessoNfe
        );
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }*/
    
    public function getNotasDownloadAutomatico($cnpj, $tpAmb) {
        $sql = "SELECT N.NRINSJURIEST,
                       N.IDAMBIENTENFE,
                       N.NRNSU,
                       N.NRACESSONFE,
                       A.DSARQXMLNSUNFE
                  FROM NSUNFE N LEFT JOIN ARQXMLNSUNFE A ON (N.NRINSJURIEST = A.NRINSJURIEST)
                                                        AND (N.IDAMBIENTENFE = A.IDAMBIENTENFE)
                                                        AND (N.NRNSU = A.NRNSU)
               WHERE N.NRINSJURIEST = :P_NRINSJURIEST
                 AND N.IDAMBIENTENFE = :P_IDAMBIENTENFE
                 AND NRACESSONFE IS NOT NULL
                 AND N.CDSTATUS IN ('100')
                 AND A.IDCONSULTADO = 'N'
                 AND NOT EXISTS (SELECT NRACESSONFE
                                   FROM NOTAFISCAL NF
                                  WHERE N.NRACESSONFE = NF.NRACESSONFE
                                    AND NF.IDENTRSAIDOP = 'E')
                 AND NOT EXISTS (SELECT NRACESSONFE
                                   FROM INTG_NOTAFISCAL INF
                                  WHERE N.NRACESSONFE = INF.NRACESSONFE)
                 AND NOT EXISTS (SELECT NRACESSONFE
                                   FROM NSUNFE NE
                                  WHERE NE.NRACESSONFE = N.NRACESSONFE
                                    AND NE.CDSERVICO IN ('110111'))";
            
        $params = array(
            "P_NRINSJURIEST"   => $cnpj,
            "P_IDAMBIENTENFE"  => $tpAmb
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function atualizaNFeBaixado($nrinsjuriest, $tpAmb, $nsu, $nrorg) {
        $sql = "UPDATE ARQXMLNSUNFE
                   SET IDCONSULTADO     = 'S'
                 WHERE NRINSJURIEST     = :P_NRINSJURIEST
                   AND IDAMBIENTENFE    = :P_IDAMBIENTENFE
                   AND TO_NUMBER(NRNSU) = TO_NUMBER(:P_NRNSU)
                   AND NRORG            = :P_NRORG";
                
        $params = [
            "P_NRINSJURIEST" => $nrinsjuriest,
            "P_IDAMBIENTENFE" => $tpAmb,
            "P_NRNSU"        => $nsu,
            "P_NRORG"        => $nrorg];
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);    
    }
    
    public function buscaDadosNotaDestinada($chaveAcesso) {
        $sql = <<<SQL
        SELECT DECODE(NF.IDORIGDESTNF,
         'F', F.NMRAZSOCFORN, DECODE(NF.IDORIGDESTNF, 'C', C.NMRAZSOCCLIE, DECODE(NF.IDORIGDESTNF, 'F', FI .NMFILIAL, NULL))) NMRAZSOC, NF.NRINSJURNF AS NRINSJUR, DECODE(NF.IDORIGDESTNF, 'F', F.NRINSESTFORN, DECODE(NF.IDORIGDESTNF, 'C', C.NRINSESTCLIE, DECODE(NF.IDORIGDESTNF, 'L', FI .CDINSCESTA))) CDINSCESTAEMIT, NF.DTEMISSAO, '1' IDENTRSAIDOP, NF.CDFILIAL, NF.NRLANCTONF, NF.NRACESSONFE, NF.VRNOTAFISC, NF.CDINSCESTA AS CDINSCESTADEST
        FROM NOTAFISCAL NF, FORNECEDOR F, FILIAL FI, CLIENTE C
        WHERE NF.IDENTRSAIDOP = 'E'
                AND NF.CDFORNECED = F.CDFORNECED(+)
                AND NF.CDCLIENTE = C.CDCLIENTE(+)
                AND NF.CDFILIEMIT = FI.CDFILIAL(+)
                AND NF.NRACESSONFE = :P_NRACESSONFE
        UNION
        ALL
        SELECT DECODE(IT.IDORIGDESTNF,
                 'F', F.NMRAZSOCFORN, DECODE(IT.IDORIGDESTNF, 'L', FI.NMFILIAL), DECODE(IT.IDORIGDESTNF, NULL, NVL (F.NMRAZSOCFORN, FI.NMFILIAL))) NMRAZSOC, IT.NRINSJURNF NRINSJUR, DECODE(IT.IDORIGDESTNF, 'F', F.NRINSESTFORN, DECODE(IT.IDORIGDESTNF, 'L', FI.CDINSCESTA)) CDINSCESTAEMIT, IT.DTEMISSAO, '1' IDENTRSAIDOP, IT.CDFILIAL, IT.NRLANCTONF, IT.NRACESSONFE, IT.VRNOTAFISC, IT.CDINSCESTA AS CDINSCESTADEST
        FROM INTG_NOTAFISCAL IT, FORNECEDOR F, FILIAL FI
        WHERE IT.CDFORNECED = F.CDFORNECED(+)
                AND IT.CDFILIAL = FI.CDFILIAL(+)
                AND IT.NRACESSONFE = :P_NRACESSONFE
                AND NOT EXISTS 
            (SELECT NRACESSONFE
            FROM NOTAFISCAL NF
            WHERE IT.NRACESSONFE = NF.NRACESSONFE
                    AND NF.IDENTRSAIDOP = 'E')
SQL;
        $params = array(
            "P_NRACESSONFE"   => $chaveAcesso);
            
        return $this->entityManager->getConnection()->fetchAll($sql);
    }
    
   /* public function incluirNotadestinada($nrProxNotadestinada, $stInsJur, $chave, $xml, $nsu, $ultNSU, $config, $idTpEvento){
        $currentTime = new \DateTime($this->environment->getCurrentTime()->format("d-m-Y H:i:s"));
        $notadestinada = new Notadestinada();
        $notadestinada->setNrnfdestinada($nrProxNotadestinada);
        $notadestinada->setNrnsu($nsu);
        $notadestinada->setNracessonfe($chave);
        $notadestinada->setNrinsjuremit($stInsJur);
        $notadestinada->setNmrazsocemit($xml->getElementsByTagName('xNome')->item(0)->nodeValue);
        $notadestinada->setCdinscestaemit($xml->getElementsByTagName('IE')->item(0)->nodeValue);
        $notadestinada->setDtemissao(DateUtil::getDataDeString(substr($xml->getElementsByTagName('dhEmi')->item(0)->nodeValue, 0, 10), 'Y-m-d'));
        $notadestinada->setIdentrsaidnf($xml->getElementsByTagName('tpNF')->item(0)->nodeValue);
        $notadestinada->setVrnotafisc($xml->getElementsByTagName('vNF')->item(0)->nodeValue);
        $notadestinada->setCddigestvalue($xml->getElementsByTagName('digVal')->item(0)->nodeValue);
        $notadestinada->setDtrecbto(DateUtil::getDataDeString(substr($xml->getElementsByTagName('dhRecbto')->item(0)->nodeValue, 0, 10), 'Y-m-d'));
        $notadestinada->setIdstatusnfe($xml->getElementsByTagName('cSitNFe')->item(0)->nodeValue);
        $notadestinada->setIdstatusmanif('0');
        $notadestinada->setIdtipoevento($idTpEvento);
        $notadestinada->setIdambientemanif($config->tpAmb);
        $notadestinada->setSgestado('AN');
        $notadestinada->setNrnotafisc(substr($chave, 25, 9));
        $notadestinada->setCdinscesta($config->ie);
        $notadestinada->setNrinsjuriest($config->cnpj);
        $notadestinada->setNrultnsupesq($ultNSU);
        $notadestinada->setNrorg($this->environment->getNrOrgTrab());
        $notadestinada->setDtinclusao($currentTime);
        $notadestinada->setNrorginclusao($this->environment->getNrOrgTrab());
        $notadestinada->setCdoperinclusao($this->environment->getCdOperador());
        $this->entityManager->persist($notadestinada);
        $this->entityManager->flush($notadestinada);
        return $notadestinada;
    }*/
    
    public function incluirNotaDestinada($nrnfdestinada, $nrnsu, $nracessonfe, $nrinsjuremit, $nmrazsocemit, $cdinscestaemit, $dtemissao, $identrsaidnf, $vrnotafisc, $cddigestvalue, $dtrecbto, $idstatusnfe, $idstatusmanif, $idtipoevento, $dthrevento, $nrseqevento, $dscorrecaonfe, $cdfilial, $nrlanctonf, $idambientemanif, $dsobsstatusmanif, $sgestado, $cdinscesta, $nrnotafisc, $nrinsjuriest, $nrultnsupesq, $nrorg) {
        $sql = "INSERT INTO NOTADESTINADA
                 (NRNFDESTINADA,
                  NRNSU,
                  NRACESSONFE,
                  NRINSJUREMIT,
                  NMRAZSOCEMIT,
                  CDINSCESTAEMIT,
                  DTEMISSAO,
                  IDENTRSAIDNF,
                  VRNOTAFISC,
                  CDDIGESTVALUE,
                  DTRECBTO,
                  IDSTATUSNFE,
                  IDSTATUSMANIF,
                  IDTIPOEVENTO,
                  DTHREVENTO,
                  NRSEQEVENTO,
                  DSCORRECAONFE,
                  CDFILIAL,
                  NRLANCTONF,
                  IDAMBIENTEMANIF,
                  DSOBSSTATUSMANIF,
                  SGESTADO,
                  CDINSCESTA,
                  NRNOTAFISC,
                  NRINSJURIEST,
                  NRULTNSUPESQ,
                  NRORG)
              VALUES
                 (:NRNFDESTINADA,
                  :NRNSU,
                  :NRACESSONFE,
                  :NRINSJUREMIT,
                  :NMRAZSOCEMIT,
                  :CDINSCESTAEMIT,
                  :DTEMISSAO,
                  :IDENTRSAIDNF,
                  :VRNOTAFISC,
                  :CDDIGESTVALUE,
                  :DTRECBTO,
                  :IDSTATUSNFE,
                  :IDSTATUSMANIF,
                  :IDTIPOEVENTO,
                  :DTHREVENTO,
                  :NRSEQEVENTO,
                  :DSCORRECAONFE,
                  :CDFILIAL,
                  :NRLANCTONF,
                  :IDAMBIENTEMANIF,
                  :DSOBSSTATUSMANIF,
                  :SGESTADO,
                  :CDINSCESTA,
                  :NRNOTAFISC,
                  :NRINSJURIEST,
                  :NRULTNSUPESQ,
                  :NRORG)";
                  
        $params = array(
            "NRNFDESTINADA" => $nrnfdestinada,
            "NRNSU" => $nrnsu,
            "NRACESSONFE" => $nracessonfe,
            "NRINSJUREMIT" => $nrinsjuremit,
            "NMRAZSOCEMIT" => $nmrazsocemit,
            "CDINSCESTAEMIT" => $cdinscestaemit,
            "DTEMISSAO" => $dtemissao,
            "IDENTRSAIDNF" => $identrsaidnf,
            "VRNOTAFISC" => $vrnotafisc,
            "CDDIGESTVALUE" => $cddigestvalue,
            "DTRECBTO" => $dtrecbto,
            "IDSTATUSNFE" => $idstatusnfe,
            "IDSTATUSMANIF" => $idstatusmanif,
            "IDTIPOEVENTO" => $idtipoevento,
            "DTHREVENTO" => $dthrevento,
            "NRSEQEVENTO" => $nrseqevento,
            "DSCORRECAONFE" => $dscorrecaonfe,
            "CDFILIAL" => $cdfilial,
            "NRLANCTONF" => $nrlanctonf,
            "IDAMBIENTEMANIF" => $idambientemanif,
            "DSOBSSTATUSMANIF" => $dsobsstatusmanif,
            "SGESTADO" => $sgestado,
            "CDINSCESTA" => $cdinscesta,
            "NRNOTAFISC" => $nrnotafisc,
            "NRINSJURIEST" => $nrinsjuriest,
            "NRULTNSUPESQ" => $nrultnsupesq,
            "NRORG" => $nrorg);
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function atualizaNotaDestinada($idstatusnfe, $idstatusmanif, $dtrecbto, $nracessonfe) {
        
        $sql = "UPDATE NOTADESTINADA
                SET IDSTATUSNFE = :P_IDSTATUSNFE,
                IDSTATUSMANIF = NVL(:P_IDSTATUSMANIF, IDSTATUSMANIF),
                DTRECBTO = NVL(:P_DTRECBTO, DTRECBTO)
                WHERE NRACESSONFE = :P_NRACESSONFE";
                
        $params = array(
            "P_IDSTATUSNFE"     => $idstatusnfe,
            "P_IDSTATUSMANIF"   => $idstatusmanif,
            "P_DTRECBTO"        => $dtrecbto,
            "P_NRACESSONFE"     => $nracessonfe);   
            

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function salvarSincronizacaonfe($xml, $notadestinada) {  
        $data = substr($xml->getElementsByTagName('dhRecbto')->item(0)->nodeValue, 0, 19);
        $data = str_replace('T', ' ', $data);
        $data = DateUtil::getDataDeString($data, 'Y-m-d H:i:s')->format('m/d/Y H:i:s');
        $data = new \DateTime($data);
        $data->format('d/m/Y H:i:s');
        $notadestinada->setIdstatusnfe($xml->getElementsByTagName('cSitNFe')->item(0)->nodeValue);
        $notadestinada->setDtrecbto($data);

        $this->entityManager->persist($notadestinada);
        $this->entityManager->flush($notadestinada);
    }
    
    public function atualizaEventoNotadestinada($idStatusNfe, $idStatusManif, $cdStatusManif, $idTipoEvento, $dtHrEvento, $nrSeqEvento, $dsCorrecaoNfe, $nrAcessoNfe) {
        $sql = "UPDATE NOTADESTINADA
                SET IDSTATUSNFE = NVL(:IDSTATUSNFE, IDSTATUSNFE),
                IDSTATUSMANIF = NVL(:IDSTATUSMANIF, IDSTATUSMANIF),
                CDSTATUSMANIF = NVL(:CDSTATUSMANIF, CDSTATUSMANIF),
                IDTIPOEVENTO = :IDTIPOEVENTO,
                DTHREVENTO = TO_DATE(:DTHREVENTO),
                NRSEQEVENTO = :NRSEQEVENTO,
                DSCORRECAONFE = :DSCORRECAONFE
                WHERE NRACESSONFE = :NRACESSONFE";
                
        $params = array(
            "IDSTATUSNFE"   => $idStatusNfe,
            "IDSTATUSMANIF" => $idStatusManif,
            "CDSTATUSMANIF" => $cdStatusManif,
            "IDTIPOEVENTO"  => $idTipoEvento,
            "DTHREVENTO"    => $dtHrEvento,
            "NRSEQEVENTO"   => $nrSeqEvento,
            "DSCORRECAONFE" => $dsCorrecaoNfe,
            "NRACESSONFE"   => $nrAcessoNfe);            
    
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);    
    }
    
    public function getNrSeqEvento($chave) {
        $sql = "SELECT TO_CHAR(NVL(TO_NUMBER(MAX(NRSEQEVENTO)), 0) + 1)  NRSEQEVENTO
            FROM NOTADESTINADA
            WHERE NRACESSONFE = :P_NRACESSONFE";
            
          $params = array(
            "P_NRACESSONFE"   => $chave);            
    
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);   
    }
    
    /*public function atualizaEventoNotadestinada($notadestinada, $xml, $idTpEvento, $idStatusNfe, $chave, $config) {
        $data = substr($xml->getElementsByTagName('dhRegEvento')->item(0)->nodeValue, 0, 19);
        $data = str_replace('T', ' ', $data);
        $data = DateUtil::getDataDeString($data, 'Y-m-d H:i:s')->format('m/d/Y H:i:s');
        $data = new \DateTime($data);
        $data->format('d/m/Y H:i:s');
        $currentTime = new \DateTime($this->environment->getCurrentTime()->format("d-m-Y H:i:s"));
        $notadestinada->setIdstatusnfe($idStatusNfe);
        $notadestinada->setIdtipoevento($idTpEvento);
        $notadestinada->setNrseqevento($xml->getElementsByTagName('nSeqEvento')->item(0)->nodeValue);
        // $notadestinada->setIdstatusmanif($xml->getElementsByTagName('cSitConf')->item(0)->nodeValue);
        $notadestinada->setDthrevento(DateUtil::getDataDeString(substr($xml->getElementsByTagName('dhRegEvento')->item(0)->nodeValue, 0, 10), 'Y-m-d'));
        $notadestinada->setDtrecbto($data);
        $notadestinada->setNrorgultatu($this->environment->getNrOrgTrab());
        $notadestinada->setDtultatu($currentTime);
        $notadestinada->setCdoperultatu($this->environment->getCdOperador());
        $this->entityManager->persist($notadestinada);
        $this->entityManager->flush($notadestinada);
    }*/
    
    public function incluirNotadestinadaManifestada($evento, $config, $idStatusManif, $xml, $cdEmpresa, $nrOrg) {
        $nota          = $this->manifestacaoDestinatarioFactory->buscaDadosNotaDestinada($evento['chNFe'])[0];
        $notaDestinada = new Notadestinada();
        $nrProxNotadestinada = $this->novoCodigo->getCodeByTableName(self::CONTADOR_NOVOCODIGO_NOTAMANIFESTACAO, $config->nrorg);
        $notaDestinada->setNrnfdestinada($nrProxNotadestinada);

        $notaDestinada->setNrnsu('');
        $notaDestinada->setNracessonfe($evento['chNFe']);
        $notaDestinada->setNrinsjuremit($nota['NRINSJUR']);
        $notaDestinada->setNmrazsocemit($nota['NMRAZSOC']);
        $notaDestinada->setCdinscestaemit($nota['CDINSCESTAEMIT']);
        $notaDestinada->setDtemissao(DateUtil::getDataDeString($nota['DTEMISSAO'], DateUtil::FORMATO_BRASILEIRO_DATAHORA));
        $notaDestinada->setIdentrsaidnf($nota['IDENTRSAIDOP']);
        $notaDestinada->setVrnotafisc($nota['VRNOTAFISC']);
        $notaDestinada->setCddigestvalue('');
        $notaDestinada->setDtrecbto(null);
        $notaDestinada->setIdstatusnfe('1');
        $notaDestinada->setIdstatusmanif($idStatusManif);
        $notaDestinada->setIdtipoevento('3');
        $notaDestinada->setDthrevento($this->formataData($evento['dhRegEvento']));
        $notaDestinada->setNrseqevento('');
        $notaDestinada->setDscorrecaonfe('');
        $notaDestinada->setCdfilial($nota['CDFILIAL']);
        $notaDestinada->setNrlanctonf($nota['NRLANCTONF']);
        $notaDestinada->setIdambientemanif($evento['tpAmb']);
        $notaDestinada->setDsobsstatusmanif($evento['xMotivo']);
        $notaDestinada->setSgestado('AN');
        $notaDestinada->setCdinscesta($config->ie);
        $notaDestinada->setNrinsjuriest($config->cnpj); // ATUALIZAR ENTIDADE
        $notaDestinada->setNrultnsupesq(''); // ATUALIZAR ENTIDADE
        $notaDestinada->setNrnotafisc('0'.substr($evento['chNFe'], 25, 9));
        $notaDestinada->setNracessointg('');
        $notaDestinada->setNrlotemanif($this->novoCodigo->getCodeByTableName(
                self::CONTADOR_NOVOCODIGO_LOTEMANIFESTACAO . $cdEmpresa,
                $nrOrg, 1, self::TAMANHO_LOTENFE));
        $notaDestinada->setNrprotocolo(($evento['nProt']) ? $evento['nProt'] : '');
        $notaDestinada->setCdibgeestadoa('91');
        $notaDestinada->setIdassinatura(($evento['nProt']) ? 'ID'.$evento['nProt'] : '');

        $notaDestinada->setNrorg($nota['NRORG']);
        $this->entityManager->persist($notaDestinada);
        $this->entityManager->flush($notaDestinada);
        
        $this->atualizaCdStatusManif($evento["chNFe"], $evento["cStat"]);
        
        $this->gravaXmlBanco($nota['NRORG'], $xml, $nota['CDFILIAL'], $nota['NRLANCTONF'], 'A');
    }
    
    public function atualizaEventoNotadestinadaManifestada($notaDestinada, $evento, $idStatusManif, $xml, $cdEmpresa, $nrOrg) {
        $notaDestinada->setIdambientemanif($evento['tpAmb']);
        $notaDestinada->setDsobsstatusmanif($evento['xMotivo']);
        $notaDestinada->setIdstatusmanif($idStatusManif);

        $notaDestinada->setNrlotemanif($this->novoCodigo->getCodeByTableName(
                self::CONTADOR_NOVOCODIGO_LOTEMANIFESTACAO . $cdEmpresa,
                $nrOrg, 1, self::TAMANHO_LOTENFE));
        if($evento['nProt']) {
            $notaDestinada->setIdassinatura('ID'.$evento['nProt']);
        }
        $notaDestinada->setCdibgeestadoa(substr($evento['chNFe'], 0, 2));
        $notaDestinada->setNrprotocolo($evento['nProt']);
        $notaDestinada->setDthrevento($this->formataData($evento['dhRegEvento']));
        $notaDestinada->setNracessonfe($evento['chNFe']);
        $this->entityManager->persist($notaDestinada);
        $this->entityManager->flush($notaDestinada);
        
        $this->atualizaCdStatusManif($evento["chNFe"], $evento["cStat"]);
        //$this->gravaXmlBanco($notaDestinada->getNrorg(), $xml, $notaDestinada->getCdfilial(), $notaDestinada->getNrlanctonf(), 'A');
    }
    
    /**
     * Grava o arquivo xml no banco de dados (Tabela ARQXMLNFE)
     * @param $nrorg
     * @param $xml Strring
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $idtipoarqxml
     * @throws \Exception
     */
    public function gravaXmlBanco($nrorg, $xml, $cdfilial, $nrlanctonf, $idtipoarqxml)
    {
        try {
            $xml = (string)str_replace('<?xml version="1.0" encoding="UTF-8"?>', ' ', $xml);
            $xml = str_replace("\n", '', $xml);

            $arqxml = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Arqxmlnfe')->findOneBy(
                array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrlanctonf' => $nrlanctonf, 'idtipoarqxml' => $idtipoarqxml));

            if ($arqxml) {
                $arqxml->setDsarqxmlnfeClob($xml);
                $this->entityManager->persist($arqxml);
                $this->entityManager->flush($arqxml);
            } else {
                $arqxml = new Arqxmlnfe();
                $sql = "INSERT INTO ARQXMLNFE (CDFILIAL, NRLANCTONF, IDTIPOARQXML, DSARQXMLNFE_CLOB, NRORG) 
                        VALUES (:P_CDFILIAL, :P_NRLANCTONF, :P_IDTIPOARQXML, :P_DSARQXMLNFECLOB, :P_NRORG)";
                    
                $params = array(
                    "P_CDFILIAL"   => $cdfilial,
                    "P_NRLANCTONF" => $nrlanctonf,
                    "P_IDTIPOARQXML" => $idtipoarqxml,
                    "P_DSARQXMLNFECLOB" => $xml,
                    "P_NRORG"           => $nrorg);            
            
                $std = $this->entityManager->getConnection()->prepare($sql);
                $std->execute($params); 
            }
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /*public function geraXmlNfeManifestacao($nota){
        $nrorg = $nota['NRORG'];
        $cdfilial = $nota['CDFILIAL'];
        $nrlanctonf = $nota['NRLANCTONF'];

        $arqxml = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Arqxmlnfe')->findOneBy(
                array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrlanctonf' => $nrlanctonf, 'idtipoarqxml' => 'A'));
        
        $notaFiscal = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Notafiscal')->findOneBy(
                array('nrorg' => $nrorg, 'cdfilial' => $cdfilial, 'nrlanctonf' => $nrlanctonf));

        if($arqxml) {
            $filename = $this->downloadXMLManifestacaoDir . DIRECTORY_SEPARATOR . $notaFiscal->getNracessonfe() . ".xml";
            file_put_contents($filename, $arqxml->getDsarqxmlnfeClob());
        }
        else throw new \Exception("Arquivo XML da nota não encontrado.");
    
        return $filename;
    }*/
    
    /**
    *   Converte uma string Xml para um array.
    *    
    *   @param string $stringXml
    *   return  array - Array do XML
    */
    protected function converteXmlParaArray($stringXml) {
        $xml = simplexml_load_string($stringXml);
        $json = json_encode($xml);
        $array = json_decode($json,TRUE);
        return $array;
    }
    
    public function getDadosNf($chaveAcesso) {
        $sql = "SELECT CDFILIAL, NRLANCTONF, NVL(IDNFGERADAVIAXML, 'N') IDNFGERADAVIAXML
                FROM NOTAFISCAL
                WHERE NRACESSONFE = :P_NRACESSONFE";
        
        $params = array(
            "P_NRACESSONFE"   => $chaveAcesso
        );
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function deletaNotaDuplicada($chaveAcesso, $nrInsjuriEst) {
        $sql = "DELETE
            FROM NOTADESTINADA
            WHERE NRACESSONFE = :P_NRACESSONFE
            AND NRINSJURIEST <> :P_NRINSJURIEST";
        
        $params = array(
            "P_NRACESSONFE"   => $chaveAcesso,
            "P_NRINSJURIEST"  => $nrInsjuriEst
        );
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getSequencialHistoricoXML($nrAcessoNfe) {
    	$sql = "SELECT MAX(NVL(NRSEQHISTOR,0)) AS NRSEQHISTOR
		        FROM HISTXMLNFE
		       WHERE NRACESSONFE = :P_NRACESSONFE";
		 
		$params = array("P_NRACESSONFE"  => $nrAcessoNfe);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
    }
    
    public function insereHistoricoXML($nrAcessoNfe, $nrSeqHistor, $dtHistorico, $idTipoOper, $idStatus, $nrProcoloNFe, $dsOperacao, $cdFilial, $nrLanctonf, $cdOperador, $nrOrg) {
    	$sql = "INSERT INTO HISTXMLNFE(NRACESSONFE, NRSEQHISTOR, DTHISTORICO, IDTIPOOPER,
                               IDSTATUS, NRPROTOCOLONFE, DSOPERACAO, CDFILIAL,
                             NRLANCTONF, CDOPERADOR, NRORG)
               VALUES (:NRACESSONFE, :NRSEQHISTOR, :DTHISTORICO, :IDTIPOOPER,
            	:IDSTATUS, :NRPROTOCOLONFE, :DSOPERACAO, :CDFILIAL,
            	:NRLANCTONF, :CDOPERADOR, :NRORG)";
              
              
        $params = array("NRACESSONFE"		=> $nrAcessoNfe,
						"NRSEQHISTOR"		=> $nrSeqHistor, 
						"DTHISTORICO"		=> $dtHistorico,
						"IDTIPOOPER"		=> $idTipoOper,
						"IDSTATUS"			=> $idStatus,
						"NRPROTOCOLONFE"	=> $nrProcoloNFe,
						"DSOPERACAO"		=> $dsOperacao,
						"CDFILIAL"			=> $cdFilial,
						"NRLANCTONF"		=> $nrLanctonf,
						"CDOPERADOR"		=> $cdOperador,
						"NRORG"				=> $nrOrg);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function buscaHistoricoXML($nrAcessoNfe) {
        $sql = "SELECT NRACESSONFE, NRSEQHISTOR, DTHISTORICO, IDTIPOOPER,
             IDSTATUS, NRPROTOCOLONFE, DSOPERACAO, CDFILIAL,
             NRLANCTONF, CDOPERADOR
        FROM HISTXMLNFE
       WHERE NRACESSONFE = :P_NRACESSONFE";
       
        $params = array("P_NRACESSONFE"  => $nrAcessoNfe);
       
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function getNotaSincronizada($nrAcessoNfe) {
        $sql = "SELECT NF.CDFILIAL,
             NF.NRLANCTONF,
             NF.NRACESSONFE,
             NF.CDINSCESTA
        FROM NOTAFISCAL NF
       WHERE NF.NRACESSONFE = :P_NRACESSONFE
         AND NF.IDENTRSAIDOP = 'E'";
         
        $params = array("P_NRACESSONFE"  => $nrAcessoNfe);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function relacionarNF($cdFilial, $nrLanctonf, $cdInscesta, $nrAcessoNfe) {
        $sql = "UPDATE NOTADESTINADA
         SET CDFILIAL = :P_CDFILIAL,
             NRLANCTONF = :P_NRLANCTONF,
             CDINSCESTA = :P_CDINSCESTA
       WHERE NRACESSONFE = :P_NRACESSONFE";
       
        $params = array("P_CDFILIAL"     => $cdFilial,
                        "P_NRLANCTONF"   => $nrLanctonf,
                        "P_CDINSCESTA"   => $cdInscesta,
                        "P_NRACESSONFE"  => $nrAcessoNfe);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function somenteXML($nrAcessoNfe) {
        $sql = "SELECT NRACESSONFE
            FROM INTG_NOTAFISCAL
            WHERE NRACESSONFE = :P_NRACESSONFE";
       
        $params = array("P_NRACESSONFE"  => $nrAcessoNfe);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function intg($nrAcessoNfe, $nrAcessoIntg) {
        $sql = "UPDATE NOTADESTINADA
                SET NRACESSOINTG  = :P_NRACESSOINTG
                WHERE NRACESSONFE = :P_NRACESSONFE";
                
        $params = array("P_NRACESSONFE"   => $nrAcessoNfe,
                        "P_NRACESSOINTG"  => $nrAcessoIntg);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getDiretorioIntegracao() {
        $sql = "SELECT DSDIRIMPARQXML FROM PARAMINTEGRA";
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
		return $std->fetchAll(); 
        
    }
    
    function formataData($date) {
        $dateDMY  = substr($date, 0, 10);
        $dateHIS  = substr($date, 11, 8);
        $formatedDate = explode('-', $dateDMY);
        $formatedDate = $formatedDate[0].'-'.$formatedDate[1].'-'.$formatedDate[2];
        $formatedDate.= ' '.$dateHIS;
        $formatedDate = strtotime($formatedDate);
        $formatedDate = date("d/m/Y H:i:s", $formatedDate);
        return DateUtil::getDataDeString($formatedDate, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
    }
    
    public function getCNPJCertificado($ie, $cdempresa) {
        $sql = "SELECT * FROM INSCRESTAD WHERE CDEMPRESA = :CDEMPRESA AND CDINSCESTA = :CDINSCESTA";
        
        $params = array("CDEMPRESA"       => $cdempresa,
                        "CDINSCESTA"      => $ie  
                        );
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();  
    }
    
    public function getCertificado($ie, $cnpj) {
        $sql = "SELECT DSDIRCERTIFICADO, CDSENHACERTIF FROM INSCRESTAD WHERE NRINSJURIEST = :CNPJ";
        
        $params = array("CNPJ"            => $cnpj);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }

    /**
     * zGravaFile
     * Grava os dados no diretorio das NFe
     *
     * @param  string $tpAmb     ambiente
     * @param  string $filename  nome do arquivo
     * @param  string $data      dados a serem salvos
     * @param  string $subFolder
     * @param  string $chNFe
     * @throws Exception\RuntimeException
     */
    public function zGravaFile(
        $tpAmb = '2',
        $filename = '',
        $data = '',
        $subFolder = 'temporarias',
        $chNFe = ''
    ) {
        $tmpFile = sys_get_temp_dir().DIRECTORY_SEPARATOR.'xmlNota'.uniqid().'.xml';
        file_put_contents($tmpFile, $data);
        $file = file_get_contents($tmpFile);
        $file    = base64_encode($file);

        $file = 'data: '.mime_content_type($tmpFile).';base64,'.$file;
        $lastModify = filectime($tmpFile);

        $file = array(
            "name" => $filename, 
            "lastModified" => $lastModify,
            "lastModifiedDate" => '',
            "webkitRelativePath" => '',
            "type" => '',
            "size"  => filesize($tmpFile),
            "b64File" => $file);
            
        $row['XML']    = array();
        $row['XML'][0] = $file;
        
        $subFolder .= date("Ym");
        $this->fileUploader->uploadFilesInRow($row, 'XML', $subFolder);
        unlink($tmpFile);
        
        $fileServerLink = $row['XML'][0]['path'];
        
        $subFolder = ($tpAmb == 1) ? 'producao/'.$subFolder : 'homologacao/'.$subFolder;
        
        $this->insertIntoRelacFileServer($chNFe, $subFolder, $fileServerLink);
        
        return $row['XML'][0]['path'];
    }
    
    private function insertIntoRelacFileServer($chNFe, $diretorio, $fileServerLink) {

        $sql = "INSERT
                INTO RELACFILESERVER
                  (
                    CDRELAC,
                    DIRETORIO,
                    FILESERVERLINK,
                    NRORG
                  )
                  VALUES
                  (
                    :CDRELAC,
                    :DIRETORIO,
                    :FILESERVERLINK,
                    :NRORG
                  )";
        
        $params = array(
            "CDRELAC"        => $chNFe,
            "DIRETORIO"      => $diretorio,
            "FILESERVERLINK" => $fileServerLink,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    protected function getFileExtension($file) {
        $fileName = $file['name'];
        $lastDotIndex = (strrpos($fileName, '.') ?: -1);
        return substr($fileName, $lastDotIndex + 1);
    }
    
    /*public function existeNota($chave) {
        $sql = "SELECT CDFILIAL, NRLANCTONF, CDINSCESTA
                FROM NOTAFISCAL
               WHERE NRACESSONFE = :P_NRACESSONF
                 AND IDENTRSAIDOP = 'E'";
                 
        $params = array(
            "CDRELAC"        => $chNFe,
            "DIRETORIO"      => $diretorio,
            "FILESERVERLINK" => $fileServerLink,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }*/
    
    public function existeNotaRelac($chave) {
        $sql = "SELECT * FROM RELACFILESERVER WHERE CDRELAC = :NRACESSONFE";
        
        $params = array(
            "NRACESSONFE"        => $chave,
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    /*public function getNotaDestinada($chave) {
        $sql = "SELECT * NOTADESTINADA WHERE NRACESSONFE = :NRACESSONFE";
        
        $params = array(
            "NRACESSONFE"        => $chave
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }*/
    
    public function getDefaultFilterManif($cdfilial) {
        $sql = "SELECT E.CDEMPRESA, E.NMEMPRESA, E.NMRAZSOCEMPR, I.NRINSJURIEST, I.CDINSCESTA, I.NMREPRINSEST, F.NRORG FROM EMPRESA E, FILIAL F, INSCRESTAD I
                WHERE F.CDEMPRESA = E.CDEMPRESA
                AND   F.CDFILIAL  = :P_CDFILIAL
                AND   E.NRINSJUREMPR = (REPLACE(REPLACE(REPLACE(I.NRINSJURIEST, '.', ''), '/', ''), '-', ''))";
    
        $params = array(
            "P_CDFILIAL"        => $cdfilial
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();  
    }
    
    public function maxNrNsu($nrinsjuriest, $tpAmb, $idconsultado) {
        $sql = "SELECT NVL(MAX(TO_NUMBER(NRNSU)), 0) AS MAXNRNSU
                    FROM NSUNFE
                   WHERE NRINSJURIEST = :P_NRINSJURIEST
                     AND IDAMBIENTENFE = :P_IDAMBIENTENFE
                     AND ((IDCONSULTADO = 'S') OR (:P_IDCONSULTADO = 'T'))";
                     
        $params = array(
            "P_NRINSJURIEST"        => $nrinsjuriest,
            "P_IDAMBIENTENFE"       => $tpAmb,
            "P_IDCONSULTADO"        => $idconsultado
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();                       
        
    }
    
    public function getDataHoraCerta() {
        $sql = "SELECT SYSDATE AS DATAHORACERTA,
                    TO_CHAR(SYSTIMESTAMP, 'YYYY-MM-DD\"T\"HH24:MI:SSTZH:TZM') AS HORAATUALUTC
                FROM DUAL";
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
		return $std->fetch(); 
    }
    
    public function consultaNsu($cnpj, $tpAmb, $nsu) {
        $sql = "SELECT DTINSNRNSU,
                     DTUPDNRNSU,
                     NRACESSONFE,
                     CDSTATUS,
                     DSMOTIVO,
                     DSSERVICO,
                     IDCONSULTADO,
                     DTRECBTO,
                     NRPROTOCOLO,
                     IDSTATUSNFE,
                     CDSERVICO,
                     NRSEQSERVICO
                FROM NSUNFE
                WHERE NRINSJURIEST = :P_NRINSJURIEST
                AND IDAMBIENTENFE = :P_IDAMBIENTENFE
                AND NRNSU = :P_NRNSU";
        
        $params = array(
            "P_NRINSJURIEST"        => $cnpj,
            "P_IDAMBIENTENFE"       => $tpAmb,
            "P_NRNSU"               => $nsu
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();     
    }
    
    public function insereNsuNfe($nrinsjuriest, $tpAmb, $nrnsu, $dtinsnrnsu, $dtupdnrnsu, $nracessonfe, $cdstatus, $dsmotivo, $dsservico, $idconsultado, $dtrecbto, $nrprotocolo, $idstatusnfe, $cdservico, $nrseqservico, $nrorg){
        $sql = "INSERT INTO NSUNFE
                   (NRINSJURIEST, IDAMBIENTENFE, NRNSU, DTINSNRNSU,
                    DTUPDNRNSU, NRACESSONFE, CDSTATUS, DSMOTIVO,
                    DSSERVICO, IDCONSULTADO, DTRECBTO, NRPROTOCOLO,
                    IDSTATUSNFE, CDSERVICO, NRSEQSERVICO, NRORG)
                VALUES
                   (:P_NRINSJURIEST, :P_IDAMBIENTENFE, :P_NRNSU, :P_DTINSNRNSU,
                    :P_DTUPDNRNSU, :P_NRACESSONFE, :P_CDSTATUS, :P_DSMOTIVO,
                    :P_DSSERVICO, :P_IDCONSULTADO, :P_DTRECBTO, :P_NRPROTOCOLO,
                    :P_IDSTATUSNFE, :P_CDSERVICO, :P_NRSEQSERVICO, :P_NRORG)";
                    
                
        $params = array(
            "P_NRINSJURIEST" => $nrinsjuriest,
            "P_IDAMBIENTENFE" => $tpAmb,
            "P_NRNSU" => $nrnsu,
            "P_DTINSNRNSU" => $dtinsnrnsu,
            "P_DTUPDNRNSU" => $dtupdnrnsu,
            "P_NRACESSONFE" => $nracessonfe,
            "P_CDSTATUS" => $cdstatus,
            "P_DSMOTIVO" => $dsmotivo,
            "P_DSSERVICO" => $dsservico,
            "P_IDCONSULTADO" => $idconsultado,
            "P_DTRECBTO" => $dtrecbto,
            "P_NRPROTOCOLO" => $nrprotocolo,
            "P_IDSTATUSNFE" => $idstatusnfe,
            "P_CDSERVICO" => $cdservico,
            "P_NRSEQSERVICO" => $nrseqservico,
            "P_NRORG" => $nrorg
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function atualizaIdConsultado($idconsultado, $nrinsjuriest, $tpAmb, $nrnsu) {
        $sql = "UPDATE NSUNFE
                SET IDCONSULTADO        = :P_IDCONSULTADO 
                WHERE NRINSJURIEST      = :P_NRINSJURIEST
                AND IDAMBIENTENFE       = :P_IDAMBIENTENFE
                AND TO_NUMBER(NRNSU)    >= TO_NUMBER(:P_NRNSU)";
                
        $params = array(
            "P_IDCONSULTADO" => $idconsultado,
            "P_NRINSJURIEST" => $nrinsjuriest,
            "P_IDAMBIENTENFE" => $tpAmb,
            "P_NRNSU" => $nrnsu);
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);    
    }
    
    /*public function atualizaNDownloads($cnpj, $chave, $nrnsu) {

        $sql = "UPDATE NOTADESTINADA SET NRDOWNLOADXML = (NVL(NRDOWNLOADXML,0) + 1) WHERE NRACESSONFE = :P_NRACESSONFE";
        
        $params = array(
            "P_NRACESSONFE"         => $chave
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }*/
    
    public function atualizaNsuNfe($dtinsnrnsu, $dtupdnrnsu, $nracessonfe, $cdstatus, $dsmotivo, $dsservico, $idconsultado, $dtrecebto, $nrprotocolo, $idstatusnfe, $cdservico, $nrseqservico, $nrinsjuriest, $tpamb, $nrnsu) {
        $sql = "UPDATE NSUNFE
                SET DTINSNRNSU      = NVL(:P_DTINSNRNSU, DTINSNRNSU),
                    DTUPDNRNSU      = NVL(:P_DTUPDNRNSU, DTUPDNRNSU),
                    NRACESSONFE     = NVL(:P_NRACESSONFE, NRACESSONFE),
                    CDSTATUS        = NVL(:P_CDSTATUS, CDSTATUS),
                    DSMOTIVO        = NVL(:P_DSMOTIVO, DSMOTIVO),
                    DSSERVICO       = NVL(:P_DSSERVICO, DSSERVICO),
                    IDCONSULTADO    = NVL(:P_IDCONSULTADO, IDCONSULTADO),
                    DTRECBTO        = NVL(:P_DTRECBTO, DTRECBTO),
                    NRPROTOCOLO     = NVL(:P_NRPROTOCOLO, NRPROTOCOLO),
                    IDSTATUSNFE     = NVL(:P_IDSTATUSNFE, IDSTATUSNFE),
                    CDSERVICO       = NVL(:P_CDSERVICO, CDSERVICO),
                    NRSEQSERVICO    = NVL(:P_NRSEQSERVICO, NRSEQSERVICO)
                WHERE NRINSJURIEST  = :P_NRINSJURIEST 
                  AND IDAMBIENTENFE = :P_IDAMBIENTENFE
                  AND NRNSU         = :P_NRNSU";
                
        $params = array(
            "P_DTINSNRNSU"      => $dtinsnrnsu,
            "P_DTUPDNRNSU"      => $dtupdnrnsu,
            "P_NRACESSONFE"     => $nracessonfe,
            "P_CDSTATUS"        => $cdstatus,
            "P_DSMOTIVO"        => $dsmotivo,
            "P_DSSERVICO"       => $dsservico,
            "P_IDCONSULTADO"    => $idconsultado,
            "P_DTRECBTO"        => $dtrecebto,
            "P_NRPROTOCOLO"     => $nrprotocolo,
            "P_IDSTATUSNFE"     => $idstatusnfe,
            "P_CDSERVICO"       => $cdservico,
            "P_NRSEQSERVICO"    => $nrseqservico,
            "P_NRINSJURIEST"    => $nrinsjuriest,
            "P_IDAMBIENTENFE"   => $tpamb,
            "P_NRNSU"           => $nrnsu
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function existeXmlNsu($nrinsjuriest, $tpamb, $nrnsu) {
        $sql = "SELECT NRINSJURIEST,
                IDAMBIENTENFE,
                NRNSU,
                DSARQXMLNSUNFE
                FROM ARQXMLNSUNFE
                WHERE NRINSJURIEST = :P_NRINSJURIEST
                AND IDAMBIENTENFE = :P_IDAMBIENTENFE
                AND NRNSU = :P_NRNSU";
                
        $params = array(
            "P_NRINSJURIEST"    => $nrinsjuriest,
            "P_IDAMBIENTENFE"   => $tpamb,
            "P_NRNSU"           => $nrnsu
        );    
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();  
    }
    
    public function insereXmlNsu($nrinsjuriest, $tpamb, $nrnsu, $xml, $nrorg) {
        $sql = "INSERT INTO ARQXMLNSUNFE (NRINSJURIEST, IDAMBIENTENFE, NRNSU, DSARQXMLNSUNFE, NRORG) VALUES (:P_NRINSJURIEST, :P_IDAMBIENTENFE, :P_NRNSU, :P_DSARQXMLNSUNFE, :P_NRORG)";
                
        $params = array(
            "P_NRINSJURIEST"    => $nrinsjuriest,
            "P_IDAMBIENTENFE"   => $tpamb,
            "P_NRNSU"           => $nrnsu,
            "P_DSARQXMLNSUNFE"  => $xml,
            "P_NRORG"           => $nrorg
        );    
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getNotasCiencia($nrinsjuriest, $tpamb) {
        $sql = "SELECT DISTINCT NRACESSONFE
                FROM NSUNFE NA
               WHERE NA.NRACESSONFE IS NOT NULL
                 AND NA.NRINSJURIEST = :P_NRINSJURIEST
                 AND NA.IDAMBIENTENFE = :P_IDAMBIENTENFE
                 AND NOT EXISTS (SELECT NRACESSONFE
                                   FROM NSUNFE NB
                                  WHERE NA.NRACESSONFE = NB.NRACESSONFE
                                    AND CDSERVICO IN ('110111', '210210'))
                 AND EXISTS (SELECT *
                               FROM NOTADESTINADA ND
                              WHERE NA.NRACESSONFE = ND.NRACESSONFE
                                AND ND.IDSTATUSMANIF <> '4'
              
                                  AND ((ND.CDSTATUSMANIF IS NULL) OR (ND.CDSTATUSMANIF NOT IN ('573', '575', '596', '650', '655'))))";
        
        $params = array(
            "P_NRINSJURIEST"    => $nrinsjuriest,
            "P_IDAMBIENTENFE"   => $tpamb
        );    
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function getCodLista($nrLista) {
        $sql = "SELECT CDLISTA
                    FROM LISTA
                   WHERE NRLISTA = :P_NRLISTA
                   ORDER BY CDLISTA";
                   
        $params = array(
            "P_NRLISTA"    => $nrLista
        );    
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function deletaLista($nrseqlista) {
        $sql = "DELETE
                FROM LISTA
               WHERE NRLISTA = :P_NRLISTA";
               
        $params = array(
            "P_NRLISTA"    => $nrseqlista
        );    
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function atualizaCdStatusManif($chave, $cdStatusManif) {
        $sql = "UPDATE NOTADESTINADA SET CDSTATUSMANIF = :P_CDSTATUSMANIF WHERE NRACESSONFE = :P_NRACESSONFE";
        
        $params = array(
            "P_NRACESSONFE"    => $chave,
            "P_CDSTATUSMANIF"  => $cdStatusManif
        );    
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getEmpresasAgendamento() {
        $sql = "SELECT DISTINCT CDEMPRESA, NRINSJURIEST
                FROM (SELECT CDEMPRESA, REPLACE(REPLACE(REPLACE(NRINSJURIEST, '.', ''), '/', ''), '-', '') AS NRINSJURIEST
                        FROM INSCRESTAD
                       WHERE DSDIRCERTIFICADO IS NOT NULL                          
                         AND CDSENHACERTIF IS NOT NULL
                       UNION ALL
                      SELECT F.CDEMPRESA, REPLACE(REPLACE(REPLACE(F.NRINSJURFILI, '.', ''), '/', ''), '-', '') AS NRINSJURIEST
                        FROM FILIAL F, INSCRESTAD I
                       WHERE SUBSTR(REPLACE(REPLACE(REPLACE(F.NRINSJURFILI, '.', ''), '/', ''), '-', ''), 1, 8) = SUBSTR(REPLACE(REPLACE(REPLACE(I.NRINSJURIEST, '.', ''), '/', ''), '-', ''), 1, 8)
                         AND I.DSDIRCERTIFICADO IS NOT NULL                          
                         AND I.CDSENHACERTIF IS NOT NULL
                         AND F.CDEMPRESA = I.CDEMPRESA)                
                ORDER BY CDEMPRESA, NRINSJURIEST";
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }
    
    public function setAgendamentoManifestacao($nrorg, $useVpd, $newEntityManager, $newFileUploader) {
        if($useVpd) {
            $this->entityManager = $newEntityManager;
            $this->novoCodigo = new NewCode($newEntityManager->getConnection());
        }
        if($nrorg > 1) $this->fileUploader  = $newFileUploader;
        
        $this->environment->setNrOrg($nrorg);
        $this->environment->setNrOrgTrab($nrorg);
        $this->environment->setCurrentUserId('AUTOMATICO');
    }
    
    public function checkLastSinc($cnpj, $tpAmb) {
        $sql = "SELECT DTSINCRONIZACAO,
                       NRULTIMONSU,
                       CASE WHEN (DTSINCRONIZACAO + 1/24) <= SYSDATE THEN 0 ELSE 1 END BOCONSULTA,
                       ABS(TRUNC((((SYSDATE - DTSINCRONIZACAO) * 24) * 60) - 61)) AS MINRESTANTES
                  FROM SINCRONIZACAONFE
                 WHERE NRINSJURIEST = :P_NRINSJURIEST
                   AND IDAMBIENTENFE = :P_IDAMBIENTENFE
                 ORDER BY DTSINCRONIZACAO DESC";
        
        $params = ["P_NRINSJURIEST"  => $cnpj,
                   "P_IDAMBIENTENFE" => $tpAmb];
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch(); 
    }
    
    public function selecaoNFManifestar($nrinsjuriest, $idambientenfe) {
        $sql = "SELECT DISTINCT NRACESSONFE
                    FROM NSUNFE NA
                    WHERE NA.NRACESSONFE IS NOT NULL
                      AND NA.NRINSJURIEST = :P_NRINSJURIEST
                      AND NA.IDAMBIENTENFE = :P_IDAMBIENTENFE
                      AND NA.DTRECBTO BETWEEN (SYSDATE - 90) AND SYSDATE
                      AND NOT EXISTS
                        (SELECT NRACESSONFE
                         FROM NSUNFE NB
                         WHERE NA.NRACESSONFE = NB.NRACESSONFE
                           AND NB.CDSTATUS IN ('100',
                                               '135',
                                               '136'))
                      AND EXISTS
                        (SELECT *
                         FROM NOTADESTINADA ND
                         WHERE NA.NRACESSONFE = ND.NRACESSONFE
                           AND ND.IDSTATUSMANIF NOT IN ('1',
                                                        '2',
                                                        '3',
                                                        '4'))";
        $params = ["P_NRINSJURIEST"   => $nrinsjuriest,
                   "P_IDAMBIENTENFE"  => $idambientenfe];
        
        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
  
}