<?php
namespace Teknisa\OPL\Services;

use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class Opl00800MigracaoRupturaOPLService {
  protected $connection;
  protected $newCode;
    
  public function __construct($entityManager, $environment, $newCode) {
    $this->entityManager  = $entityManager;
    $this->connection     = $entityManager->getConnection();
    $this->environment    = $environment;
    $this->newCode        = $newCode;
  }

  public function realizaCommit() {
    $this->entityManager->commit();
  }
    
  public function realizaRollback() {
    $this->entityManager->rollBack();
  }
    
  public function beginTransaction() {
    $this->entityManager->beginTransaction();
  }

  public function procedureGeraNovoCodigo($tabela) {
    try {
        //return $this->newCode->getCodeByTableName($p_contador, $this->environment->getNrorgTrab(), 1, 10, false);
        return $this->newCode->getCodeByTableName($tabela, $this->environment->getNrOrg());
    } catch(Exception $e) {
        throw $e;
    }
  }
  
  public function retornaCentralCompras($cdfilial) {
    $select = "SELECT NVL(FILIAL.CDFILCENTCMP,FILIAL.CDFILIAL) AS CDFILCENTCMP FROM FILIAL WHERE FILIAL.CDFILIAL = :CDFILIAL AND FILIAL.NRORG = :NRORG";
        
    $params = [
      "CDFILIAL" => $cdfilial == null ? $this->environment->getCdFilial() : $cdfilial,
      "NRORG" => $this->environment->getNrOrg()
    ];
        
    $cdfilcentcmp = self::executaSql($select, $params);
        
    return is_array($cdfilcentcmp) && count($cdfilcentcmp) ? $cdfilcentcmp[0]['CDFILCENTCMP'] : false;
  }

  public function adqFornecedor($row) {
    $param = array();
    $param["P_CDFORNECED"] = $row['CDFORNECED'];
    
    $query = "SELECT IDGERAFDTENT, IDTPFORPGTFO, CDFORMPGTO, NRDIAPGFIXFO, NRPARCPGTOFO, IDUTDIAPGTFO FROM FORNECEDOR WHERE CDFORNECED = :P_CDFORNECED";
                        
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);

      $return = $std->fetchAll();
      if(isset($return[0]))
        return $return[0];
    } catch(Exception $e) {
      throw $e;
    }
  }

  public function acmInsAFMigra($row, $novoCodigoAF) {
    $param = array();
    $param['CDFILDEST']     = $row['CDFILDEST'];
    $param['CDFILLANC']     = $row['CDFILLANC'];
    $param['NRAUTOFORN']    = $row['NRAUTOFORN'];
    $param['P_NRAUTOFORN']  = $novoCodigoAF;

    $query = "INSERT INTO AUTOFORNE 
              (CDFILDEST, CDFILLANC, NRAUTOFORN, CDOPERADOR, CDFORNECED, DTAUTOFORN, CDFORMPGTO, IDTPFORPGTAF,
              IDTIPOEMITAF, CDEMPRESA, IDAFOPERLOG, CDFILIOPELOG, NRDIAPGFIXAF, NRPARCPGTOAF, IDUTDIAPGTAF, IDTIPOLANCAF)
              SELECT A.CDFILDEST, A.CDFILLANC, :P_NRAUTOFORN,  A.CDOPERADOR, A.CDFORNECED, TO_DATE(TRUNC(SYSDATE)),
                A.CDFORMPGTO, A.IDTPFORPGTAF, A.IDTIPOEMITAF, A.CDEMPRESA, A.IDAFOPERLOG,  A.CDFILIOPELOG, A.NRDIAPGFIXAF,
                A.NRPARCPGTOAF, A.IDUTDIAPGTAF, A.IDTIPOLANCAF FROM AUTOFORNE A WHERE A.CDFILDEST
                  = :CDFILDEST AND A.CDFILLANC    = :CDFILLANC AND A.NRAUTOFORN   = :NRAUTOFORN";

    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);
    } catch(Exception $e) {
      throw $e;
    }
  }

  public function acmInsPrazoPgtAF($row, $novoCodigoAF) {
    $param = array();
    $param['P_CDFILDEST']   = $row['CDFILDEST'];
    $param['P_CDFILLANC']   = $row['CDFILLANC'];
    $param['P_NRAUTOFORN']  = $novoCodigoAF;
    $param['P_CDFORNECED']  = $row['CDFORNECED'];

    $query = "INSERT INTO PRAZOPGTAF (CDFILDEST, CDFILLANC, NRAUTOFORN, NRDIASPGTOAF, VRPERCPGTOAF)
              (SELECT :P_CDFILDEST, :P_CDFILLANC, :P_NRAUTOFORN, NRDIASPGTOFO, VRPERCPGTOFO
              FROM PRAZOPGTFORN
              WHERE CDFORNECED = :P_CDFORNECED)";
              
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);
    } catch(Exception $e) {
      throw $e;
    }
  }

  public function adqPesqItmAfExistente($row) {
    $param = array();
    $param['P_CDFILDEST']   = $row['CDFILDEST'];
    $param['P_CDFILLANC']   = $row['CDFILLANC'];
    $param['P_NRAUTOFORN']  = $row['NEW_NRAUTOFORN'];
    $param['P_CDMARCA']     = $row['CDMARCA'];
    $param['P_CDPRODUTO']   = $row['CDARVPROD'];
    $param['P_DTPROGENTAF'] = $row['NEW_DTPROGENTAF'];

    $query = "SELECT I.CDFILDEST, I.CDFILLANC, I.NRAUTOFORN, I.QTPROGENTAF
                FROM ITEMAUTF I, AUTOFORNE A
              WHERE I.CDFILDEST   = :P_CDFILDEST
                AND I.CDFILLANC   = :P_CDFILLANC
                AND I.NRAUTOFORN  = :P_NRAUTOFORN
                AND I.CDMARCA     = :P_CDMARCA
                AND I.CDPRODUTO   = :P_CDPRODUTO
                AND I.DTPROGENTAF = :P_DTPROGENTAF
                AND I.CDFILLANC   = A.CDFILLANC
                AND I.CDFILDEST   = A.CDFILDEST
                AND I.NRAUTOFORN  = A.NRAUTOFORN
                AND A.IDBAIXAAF IS NULL
                AND NVL(I.IDITAFMIGRADA, 'N') = 'N'";
                        
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);

      $return = $std->fetchAll();
      if(isset($return[0]))
        return $return[0];
    } catch(Exception $e) {
      throw $e;
    }
  }

  public function acmInsItmAFMigra($row) {
    $param = array();
    $param['CDFILDEST']     = $row['CDFILDEST'];
    $param['CDFILLANC']     = $row['CDFILLANC'];
    $param['NRAUTOFORN']    = $row['NRAUTOFORN'];
    $param['CDMARCA']       = $row['CDMARCA'];
    $param['CDPRODUTO']     = $row['CDARVPROD'];
    $param['DTPROGENTAF']   = substr($row['DTPROGENTAF'], 0, 10);
    $param['P_NRAUTOFORN']  = $row['NEW_NRAUTOFORN'];
    $param['P_DTPROGENTAF'] = $row['NEW_DTPROGENTAF'];

    $query = "INSERT INTO ITEMAUTF (CDFILDEST, CDFILLANC, NRAUTOFORN, CDMARCA, CDPRODUTO, DTPROGENTAF, QTPROGENTAF,
                  VRPREUNIAF, CDFILDESTORIG, CDFILLANCORIG, NRAUTOFORNORIG, DTENTRITMSOL,
                CDPRODUTOORIG, CDMARCAORIG, DTPROGENTAFORIG)
              SELECT I.CDFILDEST, I.CDFILLANC, :P_NRAUTOFORN, I.CDMARCA, I.CDPRODUTO, :P_DTPROGENTAF, I.QTPROGENTAF,
              I.VRPREUNIAF, I.CDFILDESTORIG, I.CDFILLANCORIG, I.NRAUTOFORN, I.DTENTRITMSOL,
              I.CDPRODUTOORIG, I.CDMARCAORIG, I.DTPROGENTAF
              FROM ITEMAUTF I
              WHERE I.CDFILDEST   = :CDFILDEST
              AND I.CDFILLANC     = :CDFILLANC
              AND I.NRAUTOFORN    = :NRAUTOFORN
              AND I.CDMARCA       = :CDMARCA
              AND I.CDPRODUTO     = :CDPRODUTO
              AND I.DTPROGENTAF   = :DTPROGENTAF";
  
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);
    } catch(Exception $e) {
      throw $e;
    }
  }

  public function acmUpdItmAFMigra($row) {
    $param = array();
    $param['P_CDFILDEST']   = $row['CDFILDEST'];
    $param['P_CDFILLANC']   = $row['CDFILLANC'];
    $param['P_CDMARCA']     = $row['CDMARCA'];
    $param['P_CDPRODUTO']   = $row['CDARVPROD'];
    $param['P_QTPROGENTAF'] = $row['QTPROGENTAF'];
    $param['P_NRAUTOFORN']  = $row['NEW_NRAUTOFORN'];
    $param['P_DTPROGENTAF'] = $row['NEW_DTPROGENTAF'];

    $query = "UPDATE ITEMAUTF
                SET QTPROGENTAF = QTPROGENTAF + :P_QTPROGENTAF
              WHERE CDFILDEST   = :P_CDFILDEST
                AND CDFILLANC   = :P_CDFILLANC
                AND NRAUTOFORN  = :P_NRAUTOFORN
                AND CDMARCA     = :P_CDMARCA
                AND CDPRODUTO   = :P_CDPRODUTO
                AND DTPROGENTAF = :P_DTPROGENTAF";
  
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);
    } catch(Exception $e) {
      throw $e;
    }
  }

  public function acmUpdItmAfOrig($row) {
    $param = array();
    $param['P_IDITEMIGRADO']  = 'S';
    $param['P_IDBAIXITMAF']   = '3';
    $param['P_CDFILDEST']     = $row['CDFILDEST'];
    $param['P_CDFILLANC']     = $row['CDFILLANC'];
    $param['P_NRAUTOFORN']    = $row['NRAUTOFORN'];
    $param['P_CDMARCA']       = $row['CDMARCA'];
    $param['P_CDPRODUTO']     = $row['CDARVPROD'];
    $param['P_DTPROGENTAF']   = substr($row['DTPROGENTAF'], 0, 10);

    $query = "UPDATE ITEMAUTF
                SET IDITAFMIGRADA  = :P_IDITEMIGRADO,
                    IDBAIXITMAF    = :P_IDBAIXITMAF,
                    IDITEMRUPTURA  = 'N',
                    NRSEQRUPTURAOL = NULL
              WHERE CDFILDEST   = :P_CDFILDEST
                AND CDFILLANC   = :P_CDFILLANC
                AND NRAUTOFORN  = :P_NRAUTOFORN
                AND CDMARCA     = :P_CDMARCA
                AND CDPRODUTO   = :P_CDPRODUTO
                AND DTPROGENTAF = :P_DTPROGENTAF";
                
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function adqVerifCtPag($row) {
    $param = array();
    $param['P_CDFILDEST']   = $row['CDFILDEST'];
    $param['P_CDFILLANC']   = $row['CDFILLANC'];
    $param['P_NRAUTOFORN']  = $row['NRAUTOFORN'];

    $query = "SELECT COUNT( DISTINCT PROD.CDTPCTPAGAR) CONTA
                FROM PRODUTO PROD, ITEMAUTF,
                     (SELECT P.CDPRODUTO, P.CDARVPROD,
                             (SELECT NVL(MAX(PRODUTO.CDARVPROD), 'W')
                                FROM PRODUTO
                               WHERE PRODUTO.CDTPCTPAGAR IS NOT NULL
                                   AND PRODUTO.CDARVPROD = SUBSTR(P.CDARVPROD, 1, LENGTH(PRODUTO.CDARVPROD))) PRODTPCTPAGAR
                        FROM ITEMAUTF, PRODUTO P
                       WHERE ITEMAUTF.CDPRODUTO  = P.CDPRODUTO
                         AND ITEMAUTF.CDFILDEST  = :P_CDFILDEST
                         AND ITEMAUTF.CDFILLANC  = :P_CDFILLANC
                         AND ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN) I
               WHERE I.PRODTPCTPAGAR     = PROD.CDARVPROD
                 AND ITEMAUTF.CDPRODUTO  = I.CDPRODUTO
                 AND ITEMAUTF.CDFILDEST  = :P_CDFILDEST
                 AND ITEMAUTF.CDFILLANC  = :P_CDFILLANC
                 AND ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN";
                
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);

      $return = $std->fetchAll();
      if(isset($return[0]))
        return $return[0];
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function acmUpdateTpCtPagar($row) {
    $param = array();
    $param['P_CDFILDEST']   = $row['CDFILDEST'];
    $param['P_CDFILLANC']   = $row['CDFILLANC'];
    $param['P_NRAUTOFORN']  = $row['NRAUTOFORN'];

    $query = "UPDATE AUTOFORNE
              SET CDTPCTPAGAR = (SELECT
                                    PRODUTO.CDTPCTPAGAR
                                 FROM
                                    PRODUTO,
                                    (SELECT
                                        MIN(CDPRODUTO) AS CDPRODUTO
                                     FROM
                                        (SELECT
                                            CDPRODUTO,
                                            MAX(QTPROGENTAF*VRPREUNIAF)
                                         FROM
                                            ITEMAUTF
                                         WHERE
                                              ITEMAUTF.CDFILDEST  = :P_CDFILDEST AND
                                              ITEMAUTF.CDFILLANC  = :P_CDFILLANC AND
                                              ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN
                                         GROUP BY
                                            CDPRODUTO
                                            HAVING
                                               MAX(QTPROGENTAF*VRPREUNIAF) =
                                               (SELECT
                                                   MAX(QTPROGENTAF*VRPREUNIAF)
                                                FROM
                                                   ITEMAUTF
                                                WHERE
                                                     ITEMAUTF.CDFILDEST  = :P_CDFILDEST   AND
                                                     ITEMAUTF.CDFILLANC  = :P_CDFILLANC   AND
                                                     ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN))) ITEMAUTF
                                 WHERE
                                      PRODUTO.CDPRODUTO = (SELECT MAX(PROD.CDPRODUTO)
                                                         FROM PRODUTO PROD
                                                         WHERE (INSTR(ITEMAUTF.CDPRODUTO,PROD.CDPRODUTO) = 1) AND
              PROD.CDTPCTPAGAR IS NOT NULL))
              WHERE
                 AUTOFORNE.CDFILDEST  = :P_CDFILDEST  AND
                 AUTOFORNE.CDFILLANC  = :P_CDFILLANC  AND
                 AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN AND
                 AUTOFORNE.CDTPCTPAGAR IS NULL";
                
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function excluiNFContaPagar($cdfildest, $cdfillanc, $nrautoforn) {
    $procedure = new StoredProcedure($this->connection, 'EXCLUI_CONTAPAGAR_PREVISTA');
    $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $cdfildest, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, $cdfillanc, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_NRAUTOFORN',Param::PARAM_INPUT, $nrautoforn,Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_RESULTADO', Param::PARAM_OUTPUT));
    $response = $procedure->execute();
    return false;
  }
    
  public function incluiNFContaPagar($cdfildest, $cdfillanc, $nrautoforn) {
    $procedure = new StoredProcedure($this->connection, 'GERA_CONTAPAGAR_PREVISTA');
    $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $cdfildest, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, $cdfillanc, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_NRAUTOFORN',Param::PARAM_INPUT, $nrautoforn,Param::PARAM_TYPE_STR));
    $response = $procedure->execute();
    return false;
  }
    
  public function fechamentoProcAF($cdfildest, $cdfillanc, $nrautoforn) {
    $procedure = new StoredProcedure($this->connection, 'FECHAMENTO_AF');
    $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $cdfildest, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_CDFILLANC', Param::PARAM_INPUT, $cdfillanc, Param::PARAM_TYPE_STR));
    $procedure->addParam(new Param('P_NRAUTOFORN',Param::PARAM_INPUT, $nrautoforn,Param::PARAM_TYPE_STR));
    $response = $procedure->execute();
    return false;
  }
  
  public function adqAfOriginal($row) {
    $param = array();
    $param['P_CDFILDEST']   = $row['CDFILDEST'];
    $param['P_CDFILLANC']   = $row['CDFILLANC'];
    $param['P_NRAUTOFORN']  = $row['NRAUTOFORN'];

    $query = "SELECT COUNT(*) ITMBAIXADO 
        FROM AUTOFORNE A, ITEMAUTF I
       WHERE A.CDFILDEST  = :P_CDFILDEST
         AND A.CDFILLANC  = :P_CDFILLANC
         AND A.NRAUTOFORN = :P_NRAUTOFORN
         AND A.CDFILDEST  = I.CDFILDEST
         AND A.CDFILLANC  = I.CDFILLANC
         AND A.NRAUTOFORN = I.NRAUTOFORN
         AND NVL(I.IDBAIXITMAF,'1') = '1'";
                
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);

      $return = $std->fetchAll();
      if(isset($return[0]))
        return $return[0];
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function acmUpdBaixaAf($row) {
    $param = array();
    $param['P_CDFILDEST']   = $row['CDFILDEST'];
    $param['P_CDFILLANC']   = $row['CDFILLANC'];
    $param['P_IDBAIXAAF']   = $row['P_IDBAIXAAF'];
    $param['P_NRAUTOFORN']  = $row['NRAUTOFORN'];
    
    $query = "UPDATE AUTOFORNE
        SET IDBAIXAAF = :P_IDBAIXAAF
      WHERE CDFILDEST   = :P_CDFILDEST
        AND CDFILLANC   = :P_CDFILLANC
        AND NRAUTOFORN  = :P_NRAUTOFORN";
        
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function AJUSTE_CONTPREVISTA($row) {
    try {
      $connection = $this->connection;
      $procedure  = new StoredProcedure($connection, 'AJUSTE_CONTPREVISTA');
      $procedure->addParam(new Param('P_CDFILDEST',  Param::PARAM_INPUT,  $row['CDFILDEST'],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDFILLANC',  Param::PARAM_INPUT,  $row['CDFILLANC'],  Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT,  $row['NRAUTOFORN'], Param::PARAM_TYPE_STR));
      $procedure->execute();
    } catch(\Exception $e){
      throw $e;
    }
  }
  
  public function acmUpdItemAF($row) {
    $param = array();
    $param['OLD_CDFILDEST']   = $row['CDFILDEST'];
    $param['OLD_CDFILLANC']   = $row['CDFILLANC'];
    $param['OLD_NRAUTOFORN']  = $row['NRAUTOFORN'];
    $param['OLD_CDPRODUTO']   = $row['CDARVPROD'];
    $param['OLD_CDMARCA']     = $row['CDMARCA'];
    $param['OLD_DTPROGENTAF'] = substr($row['DTPROGENTAF'], 0, 10);
    $param['CDPRODRUP']       = null;
    $param['CDMARCARUP']      = null;
    $param['VRPREUNIRUP']     = null;
    $param['QTNOVAPROGAF']    = null;
    $param['DSJUSTITMAF']     = 'Entrega Direta (Ponto a Ponto)';
    $param['IDTIPOLANITM']    = 'M';
    $param['IDTPGERARUP']     = 'S';
    
    $query = "UPDATE ITEMAUTF SET
                QTNOVAPROGAF = :QTNOVAPROGAF,
                DSJUSTITMAF  = :DSJUSTITMAF,
                CDPRODRUP    = :CDPRODRUP,
                CDMARCARUP   = :CDMARCARUP,
                VRPREUNIRUP  = :VRPREUNIRUP,
                IDTIPOLANITM = NVL(:IDTIPOLANITM, IDTIPOLANITM),
                IDTPGERARUP  = :IDTPGERARUP
              WHERE
                CDFILDEST        = :OLD_CDFILDEST   AND
                CDFILLANC        = :OLD_CDFILLANC   AND
                NRAUTOFORN       = :OLD_NRAUTOFORN  AND
                CDMARCA          = :OLD_CDMARCA     AND
                CDPRODUTO        = :OLD_CDPRODUTO   AND
                DTPROGENTAF      = :OLD_DTPROGENTAF";
        
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function acmUpdItemAF1($row, $tipo) {
    $param = array();
    $param['OLD_CDFILDEST']   = $row['CDFILDEST'];
    $param['OLD_CDFILLANC']   = $row['CDFILLANC'];
    $param['OLD_NRAUTOFORN']  = $row['NRAUTOFORN'];
    $param['OLD_CDPRODUTO']   = $row['CDARVPROD'];
    $param['OLD_CDMARCA']     = $row['CDMARCA'];
    $param['OLD_DTPROGENTAF'] = substr($row['DTPROGENTAF'], 0, 10);
    $param['IDTPGERARUP']     = $tipo;
    $param['QTATENDPEDOL']    = null;
    
    $query = "UPDATE ITEMAUTF
         SET IDTPGERARUP    = :IDTPGERARUP,
             QTATENDPEDOL   = :QTATENDPEDOL,
             IDITMSEPARADO  = 'N'
       WHERE
        CDFILDEST        = :OLD_CDFILDEST   AND
        CDFILLANC        = :OLD_CDFILLANC   AND
        NRAUTOFORN       = :OLD_NRAUTOFORN  AND
        CDMARCA          = :OLD_CDMARCA     AND
        CDPRODUTO        = :OLD_CDPRODUTO   AND
        DTPROGENTAF      = :OLD_DTPROGENTAF";
        
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function adqParasupri($row) {
    $param = array();
    $param['P_CDFILCENTCMP'] = $row['CDFILCENTCMP'];

    $query = "SELECT IDGERASOLICMPEXT
                FROM PARASUPRI
               WHERE CDFILIAL = :P_CDFILCENTCMP";
                
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);

      $return = $std->fetchAll();
      if(isset($return[0]))
        return $return[0];
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function acmUpdItemAFDesfazer($row) {
    $param = array();
    $param['OLD_CDFILDEST']   = $row['CDFILDEST'];
    $param['OLD_CDFILLANC']   = $row['CDFILLANC'];
    $param['OLD_NRAUTOFORN']  = $row['NRAUTOFORN'];
    $param['OLD_CDPRODUTO']   = $row['CDARVPROD'];
    $param['OLD_CDMARCA']     = $row['CDMARCA'];
    $param['OLD_DTPROGENTAF'] = substr($row['DTPROGENTAF'], 0, 10);
    $param['CDPRODRUP']       = null;
    $param['CDMARCARUP']      = null;
    $param['VRPREUNIRUP']     = null;
    $param['QTNOVAPROGAF']    = null;
    $param['DSJUSTITMAF']     = null;
    $param['IDTIPOLANITM']    = null;
    $param['IDTPGERARUP']     = null;
    
    $query = "UPDATE ITEMAUTF SET
                QTNOVAPROGAF = :QTNOVAPROGAF,
                DSJUSTITMAF  = :DSJUSTITMAF,
                CDPRODRUP    = :CDPRODRUP,
                CDMARCARUP   = :CDMARCARUP,
                VRPREUNIRUP  = :VRPREUNIRUP,
                IDTIPOLANITM = NVL(:IDTIPOLANITM, IDTIPOLANITM),
                IDTPGERARUP  = :IDTPGERARUP
              WHERE
                CDFILDEST        = :OLD_CDFILDEST   AND
                CDFILLANC        = :OLD_CDFILLANC   AND
                NRAUTOFORN       = :OLD_NRAUTOFORN  AND
                CDMARCA          = :OLD_CDMARCA     AND
                CDPRODUTO        = :OLD_CDPRODUTO   AND
                DTPROGENTAF      = :OLD_DTPROGENTAF";
        
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function concluirAnalise($row) {
    try {
      $procedure = new StoredProcedure($this->connection, 'CONCLUI_MIGRACAO_OL');
      $procedure->addParam(new Param('P_CDFILIOPERLOG', Param::PARAM_INPUT, $row['P_CDFILIOPERLOG'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_NRSEQRUPTURAOL', Param::PARAM_INPUT, $row['P_NRSEQRUPTURAOL'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODINI', Param::PARAM_INPUT, $row['P_CDPRODINI'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODFIN', Param::PARAM_INPUT, $row['P_CDPRODFIN'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_MENS', Param::PARAM_INPUT, $row['P_MENS'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
      
      return $procedure->execute();
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function validaAlteraMigracaoOL($row) {
    $param = array();
    $param['P_CDFILDEST']     = $row['CDFILDEST'];
    $param['P_CDMARCA']       = $row['CDMARCA'];
    $param['P_CDPRODUTO']     = $row['CDPRODUTO'];
    $param['P_DTPROGENTAF']   = substr($row['DTPROGENTAF'], 0, 10);
    $param['P_CDPRODRUP']     = $row['CDPRODRUP'];
    $param['P_CDMARCARUP']    = $row['CDMARCARUP'];
    $param['P_CDFILCENTCMP']  = $row['CDFILCENTCMP'];
    $param['P_CDFORNOPELOG']  = $row['CDFORNECED'];
    
    $query = "SELECT VD_ALTERA_MIGRACAO_OL(:P_CDFILDEST, :P_CDMARCA, :P_CDPRODUTO, :P_DTPROGENTAF, :P_CDPRODRUP, :P_CDMARCARUP, :P_CDFILCENTCMP, :P_CDFORNOPELOG) V_ERROS FROM DUAL";
    
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);

      $return = $std->fetchAll();
      if(isset($return[0]))
        return $return[0];
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function getEstoqueProduto($row) {
    $param = array();
    $param['P_CDFILIOPELOG']  = $row['P_CDFILIOPERLOG'];
    $param['P_CDPRODUTO']     = $row['CDARVPROD'];
    $param['P_DTPROGENTAF']   = substr($row['DTPROGENTAF'], 0, 10);
    
    $query = "SELECT NVL(CALC_ESTOQ_CONS(:P_CDFILIOPELOG, :P_CDPRODUTO, :P_DTPROGENTAF), 0) ESTOQ_DISPO FROM DUAL";
    
    try {
      $std = $this->connection->prepare($query);
      $std->execute($param);

      $return = $std->fetchAll();
      if(isset($return[0]))
        return $return[0];
    } catch(Exception $e) {
      throw $e;
    }
  }
  
  public function procedureAlteraMigracaoOL($row) {
    try {
      $procedure = new StoredProcedure($this->connection, 'ALTERA_MIGRACAO_OL');
      $procedure->addParam(new Param('P_CDFILIOPERLOG', Param::PARAM_INPUT, $row['P_CDFILIOPERLOG'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_NRSEQRUPTURAOL', Param::PARAM_INPUT, $row['P_NRSEQRUPTURAOL'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODINI', Param::PARAM_INPUT, $row['P_CDPRODINI'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODFIN', Param::PARAM_INPUT, $row['P_CDPRODFIN'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_MENS', Param::PARAM_INPUT, $row['P_MENS'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDPRODRUP', Param::PARAM_INPUT, $row['CDPRODRUP'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_CDMARCARUP', Param::PARAM_INPUT, $row['CDMARCARUP'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ESTOQ_DISPO', Param::PARAM_INPUT, $this->getEstoqueProduto($row)['ESTOQ_DISPO'], Param::PARAM_TYPE_STR));
      $procedure->addParam(new Param('P_ERROS', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
      
      return $procedure->execute();
    } catch(Exception $e) {
      throw $e;
    }
  }
}