<?php
/**
 * Created by PhpStorm.
 * User: ingridmoreira
 * Date: 30/01/2015
 * Time: 12:08
 */

namespace Folha\Logic\EscalaTrabalho;


use Folha\Entities\GpeAlteescala;
use Folha\Entities\GpeVinculom;
use HCM\Util\DateUtil;

class AlteracaoEscala {
    public static function verificaExistenciaAlteracaoVinculo($nrTipoVinculo, $criterio, $dsInicio, $dsFim,
                                                              $nrOrg, $nrOrgPadrao, $dtCompetencia, $camposCadastros){
        //Busca todos os vínculos de acordo com os critérios da seleção que não estão rescindidos no período
        $vinculoRep = GpeVinculom::getRepository();
        /** @var GpeVinculom $vinculoRep */
        $vinculos   = $vinculoRep->getVinculosPorSelecaoNRescindidos($criterio, $dsInicio, $dsFim,$dtCompetencia,
                                                                     $nrTipoVinculo, $nrOrgPadrao, $nrOrg);

        try {
            foreach ($vinculos as $vinculo) {
                $whereAlteEscala = "NRORG = ?
                                AND NRVINCULOM = ?
                                AND TO_CHAR(DTINIESCALA, 'DD/MM/YYYY') = ?";

                $paramsAlteEscala = array($nrOrg, $vinculo['NRVINCULOM'], $camposCadastros['INIESCALA']);

                $alteEscala = GpeAlteescala::getObjectByWhere($whereAlteEscala, $paramsAlteEscala);

                if (is_object($alteEscala)) {
                    return true;
                }
            }
        }catch(\Exception $e){
            tk_message(tk_get_message_by_name('msg_gpe_alteescala_001_erroBuscaAlteracoes'), 'warning', true, true);
        }
    }

    public static function realizaCadastroAutomatico($nrTipoVinculo, $criterio, $dsInicio, $dsFim,
                                                     $nrOrg, $nrOrgPadrao, $dtCompetencia, $camposCadastros, $alterar = false){

        //Busca todos os vínculos de acordo com os critérios da seleção que não estão rescindidos no período
        $vinculoRep = GpeVinculom::getRepository();
        /** @var GpeVinculom $vinculoRep */
        $vinculos   = $vinculoRep->getVinculosPorSelecaoNRescindidos($criterio, $dsInicio, $dsFim,$dtCompetencia,
                                                                     $nrTipoVinculo, $nrOrgPadrao, $nrOrg);

        $db = \Setforms2\Runtime\Connection::getInstance();
        $db->getConn()->beginTransaction();
        $cdOperador = \Geral\Util\Ambiente::getCdoperador();
        $dtCorrente = DateUtil::getDataAtual();
        $cadastrou = false;
        try {
            foreach ($vinculos as $vinculo) {
                $whereAlteEscala = "NRORG = ?
                                AND NRVINCULOM = ?
                                AND TO_CHAR(DTINIESCALA, 'DD/MM/YYYY') = ?";

                $paramsAlteEscala = array($nrOrg, $vinculo['NRVINCULOM'], $camposCadastros['INIESCALA']);

                $alteEscala = GpeAlteescala::getObjectByWhere($whereAlteEscala, $paramsAlteEscala);

                if (!is_object($alteEscala)) {
                    $alteEscala = new GpeAlteescala();
                    $alteEscala->setNralteescala(\Geral\Util\NovoCodigo::geraCodigo($nrOrg, 'GPE_ALTEESCALA', 1, 10, true));
                    $alteEscala->setNrvinculom($vinculo['NRVINCULOM']);
                    $alteEscala->setNrescalatrabm($camposCadastros['ESCALA']);
                    $alteEscala->setDtiniescala(DateUtil::getDataDeString($camposCadastros['INIESCALA'], DateUtil::FORMATO_BRASILEIRO, true));

                    if (!(is_null($camposCadastros['FIMESCALA']) || $camposCadastros['FIMESCALA'] == '')) {
                        $alteEscala->setDtfimescala(DateUtil::getDataDeString($camposCadastros['FIMESCALA'], DateUtil::FORMATO_BRASILEIRO, true));
                    }

                    $alteEscala->setNrturno($camposCadastros['TURNO']);
                    $alteEscala->setNrhordiam($camposCadastros['HORARIO']);
                    $alteEscala->setNrmotivoescala($camposCadastros['MOTIVOESCALA']);
                    $alteEscala->setNrorg($nrOrg);
                    $alteEscala->setDtinclusao($dtCorrente);
                    $alteEscala->setCdoperinclusao($cdOperador);
                    $alteEscala->setNrorginclusao($nrOrg);
                    $alteEscala->setDtultatu($dtCorrente);
                    $alteEscala->setCdoperultatu($cdOperador);
                    $alteEscala->setNrorgultatu($nrOrg);

                    $alteEscala->persist();
                    $cadastrou = true;
                } elseif($alterar){
                    $alteEscala->setNrescalatrabm($camposCadastros['ESCALA']);

                    if (!(is_null($camposCadastros['FIMESCALA']) || $camposCadastros['FIMESCALA'] == '')) {
                        $alteEscala->setDtfimescala(DateUtil::getDataDeString($camposCadastros['FIMESCALA'], DateUtil::FORMATO_BRASILEIRO, true));
                    }

                    $alteEscala->setNrturno($camposCadastros['TURNO']);
                    $alteEscala->setNrhordiam($camposCadastros['HORARIO']);
                    $alteEscala->setNrmotivoescala($camposCadastros['MOTIVOESCALA']);
                    $alteEscala->setDtultatu($dtCorrente);
                    $alteEscala->setCdoperultatu($cdOperador);
                    $alteEscala->setNrorgultatu($nrOrg);
                    $alteEscala->persist();
                    $cadastrou = true;
                }
            }
            if($cadastrou){
                $connection = \Setforms2\Shared\Registry::getEntityManager();
                $connection->flush();
                $db->getConn()->commit();
            }
            tk_set_env_var('CADASTROU', $cadastrou);
        }
        catch (\Exception $e) {
            $db->rollback();
            tk_message(tk_get_message_by_name('msg_gpe_alteescala_001_erroCadastroAlteracao'), 'warning', true, true);
        }

        tk_close_window('gpe_alteescala_001', true);
    }
} 