<?php
/**
 * Created by PhpStorm.
 * User: phillipesilva
 * Date: 17/09/14
 * Time: 11:28
 */

namespace Geral\Logic\FonteDados;

use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Geral\Entities\Colunafontedados;
use Setforms2\Shared\Registry;
use Setforms2\Runtime\Connection;

class Coluna {
    public static function deleteColunas($nrLinha, $nrOrg)
    {
        $db = Connection::getInstance();
        $db->getConn()->beginTransaction();
        $manager = Registry::getEntityManager();

        try{
            $colunas = Colunafontedados::findByNrlinhafontedadosAndNrorg($nrLinha, $nrOrg);

            foreach($colunas as $coluna) {
                $coluna->delete();
            }

            $manager->flush();
            $db->getConn()->commit();
        }catch(\Exception $e){
            $db->getConn()->rollback();
            throw $e;
        }
    }

    public static function insertColumsAndParams($colunas, $params, $nrOrg, $linha, $posicao)
    {
        $db = Connection::getInstance();
        $db->getConn()->beginTransaction();
        $manager = Registry::getEntityManager();

        $dataAtual  = DateUtil::getDataAtual();
        $cdOperador = Ambiente::getCdOperador();

        try{
            self::deleteColunasSQL($linha, $nrOrg);
            ParametroSQL::deleteParametros($linha, $nrOrg);

            foreach($colunas as $coluna) {
                self::insertColuna($coluna, $nrOrg, $linha, $dataAtual, $cdOperador, $posicao);
                $posicao++;
            }

            foreach($params as $param) {
                ParametroSQL::insertParametro($param, $nrOrg, $linha, $dataAtual, $cdOperador);
            }

            $manager->flush();
            $db->getConn()->commit();
        }catch(\Exception $e){
            $db->getConn()->rollback();
            throw $e;
        }
    }

    private static function insertColuna($dsColuna, $nrOrg, $linha, $dataAtual, $cdOperador, $posicao)
    {
        $coluna = new Colunafontedados();

        $coluna->setNrcolunafontedados(NovoCodigo::geraCodigo($nrOrg, 'COLUNAFONTEDADOS', 1, 10, true, false));
        $coluna->setDscolunafontedados($dsColuna);
        $coluna->setNrlinhafontedados($linha);
        $coluna->setNrposicaoinicial($posicao);
        $coluna->setNrposicaofinal($posicao);
        $coluna->setIdativo('S');
        $coluna->setNrorg($nrOrg);
        $coluna->setNrorginclusao($nrOrg);
        $coluna->setNrorgultatu($nrOrg);
        $coluna->setDtinclusao($dataAtual);
        $coluna->setDtultatu($dataAtual);
        $coluna->setCdoperinclusao($cdOperador);
        $coluna->setCdoperultatu($cdOperador);

        $coluna->persist();
    }

    private static function deleteColunasSQL($nrLinha, $nrOrg)
    {
        $colunas = Colunafontedados::findByNrlinhafontedadosAndNrorg($nrLinha, $nrOrg);

        foreach($colunas as $coluna) {
            $coluna->delete();
        }

        Registry::getEntityManager()->flush();
    }
} 