/*
TECNISA - TECNOLOGIA EM INFORMTICA

NOME: APURAC_GESMERENDA

DESCRIO:
PROCEDIMENTO PARA APURAO DE FATURAMENTO DE CLIENTES EM FILIAIS
QUE UTILIZAM O TIPO DE CONTRATO GESTO MERENDA.

*/
IF EXISTS (SELECT *
           FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'APURAC_GESMERENDA' AND
                 TYPE = 'P' 			         AND
                 U.UID = O.UID             	AND
                 U.NAME = USER)
DROP PROCEDURE APURAC_GESMERENDA
GO
CREATE PROCEDURE APURAC_GESMERENDA
(
   @P_CDFILCENTPME VARCHAR(04),
   @P_CDFILIAL     VARCHAR(04),
   @P_CDCLIENTE    VARCHAR(21),
   @P_DATAINI      DATETIME,
   @P_DATAFIM      DATETIME,
   @P_NRAPURACFAT  VARCHAR(10),
   @P_IDGERANOTA   VARCHAR(01) = 'S',
   @P_DTSAIDAPUR   DATETIME = NULL
)
AS
   
   DECLARE @STIDUTEFEPLA VARCHAR(01);
   
   DECLARE @V_CDFILCENTPME VARCHAR(04);
   DECLARE @V_CDFILIAL     VARCHAR(04);
   DECLARE @V_CDCLIENTE    VARCHAR(21);
   DECLARE @V_CDTIPOCARD   VARCHAR(10)
   DECLARE @V_CDSERVFILI   VARCHAR(05);
   DECLARE @V_DTFATURAM    DATETIME;
   DECLARE @V_CDPRODUTO    VARCHAR(10);
   DECLARE @V_VRSERVICO    DOUBLE PRECISION;
   DECLARE @V_QTEFETSERV   DOUBLE PRECISION;
   DECLARE @V_NMSERVICO  VARCHAR(40);
   
   DECLARE @V_CDFILIALDEL   VARCHAR(04);
   DECLARE @V_NRLANCTONFDEL VARCHAR(10);
   
   DECLARE @V_CDFILCENTAPGM  VARCHAR(04);
   DECLARE @V_CDFILIALAPGM   VARCHAR(04);
   DECLARE @V_CDCLIENTEAPGM  VARCHAR(21);
   DECLARE @V_CDTIPOCARDAPGM VARCHAR(10);
   DECLARE @V_CDSERVFILIAPGM VARCHAR(05);
   DECLARE @V_CDPRODUTOAPGM  VARCHAR(10);
   DECLARE @V_NREFETIVOSAPGM DOUBLE PRECISION;
   DECLARE @V_VRPRECUNITAPGM DOUBLE PRECISION;
   
   DECLARE @V_EXISTE     DOUBLE PRECISION;
   DECLARE @V_CDSERIE      VARCHAR(05);
   DECLARE @V_TTITENSNF    DOUBLE PRECISION;
   DECLARE @V_CONTITEM     DOUBLE PRECISION;
   DECLARE @V_NRLANCTONF   VARCHAR(20);
   DECLARE @V_NRSEQITPROD  DOUBLE PRECISION;
   DECLARE @V_CDEMPRESA    VARCHAR(02);
   DECLARE @V_CDINSCMUNI   VARCHAR(16);
   DECLARE @V_CDINSCESTA   VARCHAR(16);
   DECLARE @V_IDTPIJURCLIE VARCHAR(01)
   DECLARE @V_NRINSJURCLIE VARCHAR(20);
   DECLARE @V_IDNOVANOTA   VARCHAR(01);
   DECLARE @V_PRXLANCESTQ  VARCHAR(10);
   DECLARE @V_IDUTPROPEMIS VARCHAR(01);
   DECLARE @V_CONTADOR     VARCHAR(17);
   DECLARE @V_CDARVPROD    VARCHAR(10);
   DECLARE @V_NMPRODUTO    VARCHAR(50);
   DECLARE @V_CDFILIALNF   VARCHAR(04);
BEGIN
   SELECT @STIDUTEFEPLA = MAX(SUBSTRING(DSPARAMFATFI,15,1)) 
   FROM PARAMFILIAL
   WHERE CDFILIAL = @P_CDFILCENTPME;
   SELECT @V_IDUTPROPEMIS = ISNULL(MAX(IDUTPROPEMIS),'N')
   FROM PARAMGERAL;
   
   DELETE FROM TRAB_APURAC_GM
   WHERE CDFILCENTAPGM  = @P_CDFILCENTPME
     AND CDFILIALAPGM = @P_CDFILIAL
     AND NRAPURACGM = @P_NRAPURACFAT
     AND CDCLIENTEAPGM = @P_CDCLIENTE;
   IF (@STIDUTEFEPLA IS NULL) OR (@STIDUTEFEPLA = ' ') BEGIN
      SET @STIDUTEFEPLA = 'N';
   END;
   
   DECLARE C_GDIRETA CURSOR FOR
      SELECT CDFILCENTPME, CDFILIAL, CDCLIENTE, CDTIPOCARD, CDSERVFILI, DTFATURAM, 
             CDPRODUTO,
             VRSERVICO,
             QTEFETIVOS,
             NMSERVICO                
      FROM
      (
      SELECT CDFILCENTPME, CDFILIAL, CDCLIENTE, CDTIPOCARD, CDSERVFILI, DTFATURAM, 
             CDPRODUTO,
             TEKNISA.VALOR_SERVICO_MERENDA(CDFILCENTPME, CDTIPOCARD, CDFILIAL, CDSERVFILI, CDCLIENTE, DTFATURAM) AS VRSERVICO,
             QTEFETIVOS,
             NMSERVICO                
      FROM
      (
      SELECT EFECLIFATMER.CDFILCENTPME, EFECLIFATMER.CDFILIAL,
             EFECLIFATMER.CDCLIENTE, EFECLIFATMER.CDTIPOCARD, EFECLIFATMER.CDSERVFILI, EFECLIFATMER.DTEFETFATMER AS DTFATURAM,
             ISNULL(SERVTIPOCARD.CDPRODUTO,SERVICO.CDPRODUTO) AS CDPRODUTO, SUM(EFECLIFATMER.QTCOMEFATMER) AS QTEFETIVOS, SERVICO.NMSERVICO
        FROM EFECLIFATMER, SERVCLIEPME, SERVICO, SERVTIPOCARD
       WHERE EFECLIFATMER.CDFILCENTPME = SERVCLIEPME.CDFILCENTPME
         AND EFECLIFATMER.CDFILIAL = SERVCLIEPME.CDFILIAL
         AND EFECLIFATMER.CDTIPOCARD = SERVCLIEPME.CDTIPOCARD
         AND EFECLIFATMER.CDCLIENTE = SERVCLIEPME.CDCLIENTE
         AND EFECLIFATMER.CDSERVFILI = SERVCLIEPME.CDSERVFILI
         AND SERVICO.CDFILIAL = EFECLIFATMER.CDFILIAL
         AND SERVICO.CDSERVFILI = EFECLIFATMER.CDSERVFILI
         
         AND (SERVTIPOCARD.CDFILCENTPME = SERVCLIEPME.CDFILCENTPME )
         AND (SERVTIPOCARD.CDTIPOCARD = SERVCLIEPME.CDTIPOCARD)
         AND (SERVTIPOCARD.CDSERVFILI = SERVICO.CDSERVFILI)
         
         AND EFECLIFATMER.DTEFETFATMER BETWEEN @P_DATAINI AND @P_DATAFIM
         AND @STIDUTEFEPLA = 'N'
         AND EFECLIFATMER.CDFILIAL   = @P_CDFILIAL
         AND EFECLIFATMER.CDCLIENTE  = @P_CDCLIENTE
      GROUP BY EFECLIFATMER.CDFILCENTPME, EFECLIFATMER.CDFILIAL, EFECLIFATMER.CDCLIENTE,
               EFECLIFATMER.CDTIPOCARD, EFECLIFATMER.DTEFETFATMER, EFECLIFATMER.CDSERVFILI,
               ISNULL(SERVTIPOCARD.CDPRODUTO,SERVICO.CDPRODUTO), SERVICO.NMSERVICO
        HAVING (SUM(EFECLIFATMER.QTCOMEFATMER) <> 0)
      UNION
      SELECT EFETCLIEPME.CDFILCENTPME, EFETCLIEPME.CDFILIAL,
             EFETCLIEPME.CDCLIENTE, EFETCLIEPME.CDTIPOCARD, EFETCLIEPME.CDSERVFILI, EFETCLIEPME.DTEFETCLPME AS DTFATURAM,
             ISNULL(SERVTIPOCARD.CDPRODUTO,SERVICO.CDPRODUTO) AS CDPRODUTO, SUM(EFETCLIEPME.QTCOMEFCLPME) AS QTEFETIVOS, SERVICO.NMSERVICO
        FROM EFETCLIEPME, SERVCLIEPME, SERVICO, SERVTIPOCARD
       WHERE EFETCLIEPME.CDFILCENTPME = SERVCLIEPME.CDFILCENTPME
         AND EFETCLIEPME.CDFILIAL = SERVCLIEPME.CDFILIAL
         AND EFETCLIEPME.CDTIPOCARD = SERVCLIEPME.CDTIPOCARD
         AND EFETCLIEPME.CDCLIENTE = SERVCLIEPME.CDCLIENTE
         AND EFETCLIEPME.CDSERVFILI = SERVCLIEPME.CDSERVFILI
         AND SERVICO.CDFILIAL = EFETCLIEPME.CDFILIAL
         AND SERVICO.CDSERVFILI = EFETCLIEPME.CDSERVFILI
         
         AND (SERVTIPOCARD.CDFILCENTPME = SERVCLIEPME.CDFILCENTPME )
         AND (SERVTIPOCARD.CDTIPOCARD = SERVCLIEPME.CDTIPOCARD)
         AND (SERVTIPOCARD.CDSERVFILI = SERVICO.CDSERVFILI)
         
         AND EFETCLIEPME.DTEFETCLPME BETWEEN @P_DATAINI AND @P_DATAFIM
         AND @STIDUTEFEPLA = 'S'
         AND EFETCLIEPME.CDFILIAL   = @P_CDFILIAL
         AND EFETCLIEPME.CDCLIENTE  = @P_CDCLIENTE
      GROUP BY EFETCLIEPME.CDFILCENTPME, EFETCLIEPME.CDFILIAL, EFETCLIEPME.CDCLIENTE,
               EFETCLIEPME.CDTIPOCARD, EFETCLIEPME.DTEFETCLPME, EFETCLIEPME.CDSERVFILI,
               ISNULL(SERVTIPOCARD.CDPRODUTO,SERVICO.CDPRODUTO), SERVICO.NMSERVICO
       HAVING (SUM(EFETCLIEPME.QTCOMEFCLPME) <> 0)
      ) S )S
      WHERE VRSERVICO <> 0;
   OPEN C_GDIRETA;
   FETCH NEXT FROM C_GDIRETA INTO @V_CDFILCENTPME, @V_CDFILIAL, @V_CDCLIENTE, @V_CDTIPOCARD, @V_CDSERVFILI, @V_DTFATURAM,
                                  @V_CDPRODUTO, @V_VRSERVICO, @V_QTEFETSERV, @V_NMSERVICO;
   WHILE (@@FETCH_STATUS = 0) BEGIN
      
      SELECT @V_EXISTE = COUNT(*)
        FROM TRAB_APURAC_GM
       WHERE CDFILCENTAPGM   = @P_CDFILCENTPME
         AND CDFILIALAPGM   = @V_CDFILIAL
         AND NRAPURACGM     = @P_NRAPURACFAT
         AND CDCLIENTEAPGM  = @V_CDCLIENTE
         AND CDTIPOCARDAPGM   = @V_CDTIPOCARD
         AND CDSERVFILIAPGM = @V_CDSERVFILI
         AND ROUND(VRPRECUNITAPGM,2) = ROUND(@V_VRSERVICO,2);
      IF (@V_EXISTE = 0) BEGIN
         INSERT INTO TRAB_APURAC_GM
           (CDFILCENTAPGM, CDFILIALAPGM, NRAPURACGM, CDCLIENTEAPGM,
            CDTIPOCARDAPGM, CDSERVFILIAPGM, DTAPURACFATGM, CDPRODUTOAPGM,
            NREFETIVOSAPGM, VRPRECUNITAPGM, NMSERVICOGM)
         VALUES (@P_CDFILCENTPME, @P_CDFILIAL, @P_NRAPURACFAT, @V_CDCLIENTE,
                 @V_CDTIPOCARD, @V_CDSERVFILI, @P_DATAINI, @V_CDPRODUTO,
                 @V_QTEFETSERV, @V_VRSERVICO, @V_NMSERVICO);
      END
      ELSE BEGIN
         UPDATE TRAB_APURAC_GM
            SET NREFETIVOSAPGM = NREFETIVOSAPGM + @V_QTEFETSERV
          WHERE CDFILCENTAPGM    = @P_CDFILCENTPME
            AND CDFILIALAPGM   = @V_CDFILIAL
            AND NRAPURACGM     = @P_NRAPURACFAT
            AND CDCLIENTEAPGM  = @V_CDCLIENTE
            AND CDTIPOCARDAPGM   = @V_CDTIPOCARD
            AND CDSERVFILIAPGM = @V_CDSERVFILI
            AND ROUND(VRPRECUNITAPGM,2) = ROUND(@V_VRSERVICO,2);
      END;
      IF @STIDUTEFEPLA = 'S' BEGIN
         UPDATE EFETCLIEPME
            SET CDFILIFATU  = NULL,
                NRAPURACFAT = NULL
          WHERE CDFILCENTPME = @V_CDFILCENTPME
            AND CDFILIAL    = @V_CDFILIAL
            AND CDCLIENTE   = @P_CDCLIENTE
            AND DTEFETCLPME >= @P_DATAINI;
      END 
      ELSE BEGIN
      UPDATE EFECLIFATMER
         SET CDFILIFATU  = NULL,
             NRAPURACFAT = NULL
       WHERE CDFILCENTPME = @V_CDFILCENTPME
         AND CDFILIAL = @V_CDFILIAL
         AND CDCLIENTE  = @P_CDCLIENTE
         AND DTEFETFATMER >= @P_DATAINI;
      END;
      FETCH NEXT FROM C_GDIRETA INTO @V_CDFILCENTPME, @V_CDFILIAL, @V_CDCLIENTE, @V_CDTIPOCARD, @V_CDSERVFILI, @V_DTFATURAM,
                                  @V_CDPRODUTO, @V_VRSERVICO, @V_QTEFETSERV, @V_NMSERVICO;
   END;
   CLOSE C_GDIRETA;
   DEALLOCATE C_GDIRETA;
   
   IF ISNULL(@P_IDGERANOTA,'S') <> 'N' BEGIN
      SELECT @V_IDTPIJURCLIE = MAX(IDTPIJURCLIE), @V_NRINSJURCLIE = MAX(NRINSJURCLIE)
      FROM CLIENTE
      WHERE CDCLIENTE = @P_CDCLIENTE;
      
      DECLARE CURS_NF CURSOR FOR SELECT CDFILIAL, NRLANCTONF
                                   FROM NOTAFISCAL
                                  WHERE (CDFILIAL    = @P_CDFILIAL)
                                    AND (NRAPURACFAT = @P_NRAPURACFAT)
                                    AND (CDCLIENTE   = @P_CDCLIENTE);
      OPEN CURS_NF;
      FETCH NEXT FROM CURS_NF INTO @V_CDFILIALDEL, @V_NRLANCTONFDEL;
      WHILE (@@FETCH_STATUS = 0) BEGIN
         EXEC DELETA_NF @V_CDFILIALDEL, @V_NRLANCTONFDEL;
      END;
      CLOSE CURS_NF;
      DEALLOCATE CURS_NF;
      SET @V_IDNOVANOTA = 'N';
      SET @V_CONTITEM = NULL;
      DECLARE CURS_APURAC CURSOR FOR SELECT CDFILCENTAPGM, CDFILIALAPGM,  CDCLIENTEAPGM, CDTIPOCARDAPGM,
                                            CDSERVFILIAPGM, CDPRODUTOAPGM, NREFETIVOSAPGM, VRPRECUNITAPGM,
                                            NMSERVICOGM AS NMSERVICO
                                       FROM TRAB_APURAC_GM
                                      WHERE (TRAB_APURAC_GM.CDFILCENTAPGM = @P_CDFILCENTPME)
                                        AND (TRAB_APURAC_GM.CDFILIALAPGM = @P_CDFILIAL)
                                        AND (TRAB_APURAC_GM.NRAPURACGM   = @P_NRAPURACFAT)
                                        AND (TRAB_APURAC_GM.CDCLIENTEAPGM  = @P_CDCLIENTE)
                                     ORDER BY CDFILCENTAPGM, CDFILIALAPGM, CDCLIENTEAPGM, CDTIPOCARDAPGM, CDSERVFILIAPGM
      OPEN CURS_APURAC;
      FETCH NEXT FROM CURS_APURAC INTO @V_CDFILCENTAPGM,  @V_CDFILIALAPGM,  @V_CDCLIENTEAPGM, @V_CDTIPOCARDAPGM,
                                       @V_CDSERVFILIAPGM, @V_CDPRODUTOAPGM, @V_NREFETIVOSAPGM, @V_VRPRECUNITAPGM,
                                       @V_NMSERVICO;
      WHILE (@@FETCH_STATUS = 0) BEGIN
         
         SELECT @V_CDEMPRESA = MAX(F.CDEMPRESA), @V_CDINSCESTA = MAX(F.CDINSCESTA), @V_CDINSCMUNI = MAX(F.CDINSCMUNI),
                @V_CDSERIE = MAX(F.CDSERIE), @V_TTITENSNF = ISNULL(MAX(NRMAXITNF),10)
           FROM FILIAL F LEFT JOIN SERIENF S
                              ON F.CDEMPRESA = S.CDEMPRESA
                              AND F.CDINSCESTA = S.CDINSCESTA
                              AND F.CDINSCMUNI = S.CDINSCMUNI
                              AND F.CDSERIE = S.CDSERIE
          WHERE CDFILIAL = @V_CDFILIALAPGM;
         IF @V_CONTITEM IS NULL BEGIN
            SET @V_CONTITEM = @V_TTITENSNF;
         END;
         IF @STIDUTEFEPLA = 'S' BEGIN
            UPDATE EFETCLIEPME
               SET CDFILIFATU  = @P_CDFILIAL,
                   NRAPURACFAT = @P_NRAPURACFAT
             WHERE CDFILCENTPME = @V_CDFILCENTPME
               AND CDFILIAL   = @V_CDFILIALAPGM
               AND CDCLIENTE  = @P_CDCLIENTE
               AND DTEFETCLPME BETWEEN @P_DATAINI AND @P_DATAFIM;
         END
         ELSE BEGIN
         UPDATE EFECLIFATMER
            SET CDFILIFATU  = @P_CDFILIAL,
                NRAPURACFAT = @P_NRAPURACFAT
          WHERE CDFILCENTPME = @V_CDFILCENTPME
            AND CDFILIAL   = @V_CDFILIALAPGM
            AND CDCLIENTE  = @P_CDCLIENTE
            AND DTEFETFATMER BETWEEN @P_DATAINI AND @P_DATAFIM;
         END;
         IF (@V_CONTITEM = @V_TTITENSNF) BEGIN
            
            IF (@V_IDNOVANOTA = 'S') BEGIN
               EXEC GERA_NOTAFISCAL_FECHAMENTO @V_CDFILIALAPGM, @V_NRLANCTONF;
            END;
            SET @V_IDNOVANOTA = 'S';
            SET @V_CONTITEM   = 0;
            SET @V_CONTADOR = 'LANCNF/FILIAL' + @V_CDFILIALAPGM;
            EXEC NOVO_CODIGO @V_CONTADOR, @P_SEQUENCIAL = @V_NRLANCTONF OUTPUT;
            SET @V_NRLANCTONF = SUBSTRING(@V_NRLANCTONF,11,10);
            SET @P_DTSAIDAPUR = ISNULL(@P_DTSAIDAPUR, @P_DATAFIM);
            SET @V_NRSEQITPROD = 0;
            EXEC GERA_NOTAFISCAL_CORPO @V_CDFILIALAPGM, @V_CDFILIALAPGM, @V_CDEMPRESA,
                                       '01', @V_CDSERIE, @P_CDCLIENTE, @P_DTSAIDAPUR,
                                       'FAI',  'N', @P_NRAPURACFAT,
                                       @V_NRLANCTONF,  NULL,         NULL,
                                        NULL,          NULL,         NULL,
                                        NULL,          NULL,         NULL,
                                        NULL,          NULL,         NULL,
                                        NULL,          NULL,         NULL,
                                        NULL,          @P_DATAFIM,    NULL, @P_PRXLANCTONF = @V_NRLANCTONF OUTPUT, @P_PRXLANCESTQ = @V_PRXLANCESTQ OUTPUT;
         END;
         SET @V_NRSEQITPROD = @V_NRSEQITPROD + 1;
         SET @V_CONTITEM = @V_CONTITEM + 1;
         IF @V_IDUTPROPEMIS = 'S' BEGIN
            SELECT @V_CDARVPROD = MAX(CDARVPROD), @V_NMPRODUTO = MAX(NMPRODUTO)
              FROM PRODUTO
             WHERE CDPRODUTO = @V_CDPRODUTOAPGM;
            EXEC GERA_NOTAFISCAL_PRODUTO @V_CDFILIALAPGM, @V_NRLANCTONF, @V_CDPRODUTOAPGM,
                                         @V_CDARVPROD, @V_NMPRODUTO, @V_NREFETIVOSAPGM, @V_VRPRECUNITAPGM;
         END
         ELSE BEGIN
            EXEC GERA_NOTAFISCAL_PRODUTO @V_CDFILIALAPGM, @V_NRLANCTONF, @V_CDPRODUTOAPGM,
                                         @V_CDSERVFILIAPGM, @V_NMSERVICO, @V_NREFETIVOSAPGM, @V_VRPRECUNITAPGM;
         END;
         
         SET @V_CDFILIALNF = @V_CDFILCENTAPGM;
                  
         FETCH NEXT FROM CURS_APURAC INTO @V_CDFILCENTAPGM,  @V_CDFILIALAPGM,  @V_CDCLIENTEAPGM, @V_CDTIPOCARDAPGM,
                                          @V_CDSERVFILIAPGM, @V_CDPRODUTOAPGM, @V_NREFETIVOSAPGM, @V_VRPRECUNITAPGM,
                                          @V_NMSERVICO;
      END;
      IF (@V_CONTITEM > 0) BEGIN
         EXEC GERA_NOTAFISCAL_FECHAMENTO @V_CDFILIALNF, @V_NRLANCTONF;
      END;
      CLOSE CURS_APURAC;
      DEALLOCATE CURS_APURAC;
     
      DELETE FROM TRAB_APURAC_GM
      WHERE CDFILCENTAPGM  = @P_CDFILCENTPME
        AND CDFILIALAPGM = @P_CDFILIAL
        AND NRAPURACGM    = @P_NRAPURACFAT
        AND CDCLIENTEAPGM = @P_CDCLIENTE;
      
      COMMIT TRANSACTION;
   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'APURAC_GESMERENDA','5.50.000';
GO